@@ -1,3 +1,28 @@
+/*
+   callback_func.c
+
+   Bygfoot Football Manager -- a small and simple GTK2-based
+   football management game.
+
+   http://bygfoot.sourceforge.net
+
+   Copyright (C) 2005  Gyözö Both (gyboth@bygfoot.com)
+
+   This program is free software; you can redistribute it and/or
+   modify it under the terms of the GNU General Public License
+   as published by the Free Software Foundation; either version 2
+   of the License, or (at your option) any later version.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+*/
+
 #include "callback_func.h"
 #include "callbacks.h"
 #include "cup.h"
@@ -19,6 +44,8 @@
 #include "user.h"
 #include "window.h"
 
+#include "bet.h"
+
 /** Show the users' live games. */
 void
 callback_show_next_live_game(void)
@@ -56,7 +83,7 @@ callback_show_next_live_game(void)
 		return;
 	    }
 
-    window_destroy(&window.live, TRUE);
+    window_destroy(&window.live);
 
     /* no more user games to show: end round. */
     end_week_round();
@@ -75,33 +102,6 @@ callback_player_clicked(gint idx, GdkEventButton *event)
 	return;
 
     if(event->button == 1)
-    {
-	if(selected_row == -1)
-	{
-	    selected_row = idx;
-	    return;
-	}
-
-	player_swap(current_user.tm, selected_row,
-		    current_user.tm, idx);
-	if(opt_user_int("int_opt_user_swap_adapts") == 1 &&
-	   current_user.tm->structure !=
-	   team_find_appropriate_structure(current_user.tm))
-	{
-	    team_change_structure(current_user.tm,
-				  team_find_appropriate_structure(current_user.tm));
-	    team_rearrange(current_user.tm);
-	}
-
-	game_gui_write_av_skills();
-
-	selected_row = -1;
-
-	treeview_show_user_player_list();
-	if(stat0 == STATUS_MAIN)
-	    treeview_show_next_opponent();
-    }
-    else if(event->button == 3)
     {
 	if(stat0 == STATUS_SHOW_TRANSFER_LIST)
 	{
@@ -115,10 +115,37 @@ callback_player_clicked(gint idx, GdkEventButton *event)
 	}
 	else
 	{
-	    selected_row = idx;
-	    window_show_menu_player((GdkEvent*)event);
+	    if(selected_row == -1)
+	    {
+		selected_row = idx;
+		return;
+	    }
+
+	    player_swap(current_user.tm, selected_row,
+			current_user.tm, idx);
+	    if(opt_user_int("int_opt_user_swap_adapts") == 1 &&
+	       current_user.tm->structure !=
+	       team_find_appropriate_structure(current_user.tm))
+	    {
+		team_change_structure(current_user.tm,
+				      team_find_appropriate_structure(current_user.tm));
+		team_rearrange(current_user.tm);
+	    }
+
+	    game_gui_write_av_skills();
+
+	    selected_row = -1;
+
+	    treeview_show_user_player_list();
+	    if(stat0 == STATUS_MAIN)
+		treeview_show_next_opponent();
 	}
     }
+    else if(event->button == 3)
+    {
+	selected_row = idx;
+	window_show_menu_player((GdkEvent*)event);
+    }
 
     setsav0;
 }
@@ -141,7 +168,8 @@ callback_show_last_match(gboolean start, LiveGame *lg)
 
 	gui_set_sensitive_lg_meters(FALSE);
 
-	treeview_show_game_stats(GTK_TREE_VIEW(lookup_widget(window.live, "treeview_stats")), lg);
+	treeview_show_game_stats(
+	    GTK_TREE_VIEW(lookup_widget(window.live, "treeview_stats")), lg);
     }
     else
     {
@@ -168,7 +196,7 @@ callback_show_last_match(gboolean start, LiveGame *lg)
     }
     else if(stat4 == STATUS_SHOW_LAST_MATCH_ABORT)
     {
-	window_destroy(&window.live, TRUE);
+	window_destroy(&window.live);
 	stat1 = stat2 = stat3 = stat4 = -1;
     }
     else
@@ -180,7 +208,7 @@ void
 callback_show_last_match_stats(void)
 {
     current_user.live_game.fix = 
-	fixture_from_id(current_user.live_game.fix_id);
+	fixture_from_id(current_user.live_game.fix_id, TRUE);
     
     treeview_show_game_stats(GTK_TREE_VIEW(lookup_widget(window.main, "treeview_right")),
 			     &current_user.live_game);
@@ -334,7 +362,8 @@ callback_transfer_list_user(gint button, gint idx)
     gchar buf[SMALL],
 	buf2[SMALL], buf3[SMALL];
 
-    if(button == 3)
+    if(button == 3 || 
+       (button == 1 && trans(idx).offers->len == 0))
     {
 	transfer_remove_player(idx);
 	on_button_transfers_clicked(NULL, NULL);
@@ -342,10 +371,8 @@ callback_transfer_list_user(gint button, gint idx)
     }
     else if(button == 1)
     {
-	if(trans(idx).offers->len == 0)
-	    game_gui_print_message(_("There are no offers for the player."));
-	else if(trans(idx).offers->len > 0 &&
-		transoff(idx, 0).status != TRANSFER_OFFER_ACCEPTED)
+	if(trans(idx).offers->len > 0 &&
+	   transoff(idx, 0).status != TRANSFER_OFFER_ACCEPTED)
 	    game_gui_print_message(_("There are some offers for the player which you rejected or will see next week."));
 	else
 	{
@@ -526,7 +553,8 @@ callback_offer_new_contract(gint idx)
 void
 callback_show_team(gint type)
 {
-    GtkTreeView *treeview_right = GTK_TREE_VIEW(lookup_widget(window.main, "treeview_right"));
+    GtkTreeView *treeview_right = 
+	GTK_TREE_VIEW(lookup_widget(window.main, "treeview_right"));
     const Team *tm;
     const GArray *teams = NULL;
     const GPtrArray *teamsp = NULL;
@@ -669,6 +697,9 @@ callback_show_league_stats(gint type)
 void
 callback_show_season_history(gint type)
 {
+    const SeasonStat *stat = NULL;
+    gint len = season_stats->len;
+
     switch(type)
     {
 	default:
@@ -677,29 +708,31 @@ callback_show_season_history(gint type)
 	    break;
 	case SHOW_CURRENT:
 	    stat1 = -1;
-	    stat2 = season - 2;
+	    stat2 = len - 1;
 	    break;
 	case SHOW_NEXT_LEAGUE:
+	    stat = &g_array_index(season_stats, SeasonStat, stat2);
 	    if(stat1 == -1)
 		stat1 = 0;
-	    else if(stat1 == ligs->len - 1)
+	    else if(stat1 == stat->league_champs->len - 1)
 		stat1 = -1;
 	    else
 		stat1++;
 	    break;
 	case SHOW_PREVIOUS_LEAGUE:
+	    stat = &g_array_index(season_stats, SeasonStat, stat2);
 	    if(stat1 == -1)
-		stat1 = ligs->len - 1;
+		stat1 = stat->league_champs->len - 1;
 	    else if(stat1 == 0)
 		stat1 = -1;
 	    else
 		stat1--;
 	    break;
 	case SHOW_NEXT:
-	    stat2 = (stat2 + 1) % (season - 1);
+	    stat2 = (stat2 + 1) % len;
 	    break;
 	case SHOW_PREVIOUS:
-	    stat2 = (stat2 == 0) ? season - 2 : stat2 - 1;
+	    stat2 = (stat2 == 0) ? len - 1 : stat2 - 1;
 	    break;
     }
     
@@ -762,9 +795,10 @@ callback_show_youth_academy(void)
 	attributes.on_off[PLAYER_LIST_ATTRIBUTE_ETAL] = 1;
 
     if(stat0 != STATUS_SHOW_YA)
-	game_gui_print_message(_("Right click to move players to and from the youth academy; left click for context menu."));
+	game_gui_print_message(_("Left click to move players to and from the youth academy; right click for context menu."));
 
-    treeview_show_player_list(GTK_TREE_VIEW(lookup_widget(window.main, "treeview_right")),
-			      player_get_pointers_from_array(current_user.youth_academy.players),
-			      attributes, FALSE);
+    treeview_show_player_list(
+	GTK_TREE_VIEW(lookup_widget(window.main, "treeview_right")),
+	player_get_pointers_from_array(current_user.youth_academy.players),
+	attributes, FALSE);
 }
