@@ -1,3 +1,28 @@
+/*
+   cup.c
+
+   Bygfoot Football Manager -- a small and simple GTK2-based
+   football management game.
+
+   http://bygfoot.sourceforge.net
+
+   Copyright (C) 2005  Gyözö Both (gyboth@bygfoot.com)
+
+   This program is free software; you can redistribute it and/or
+   modify it under the terms of the GNU General Public License
+   as published by the Free Software Foundation; either version 2
+   of the License, or (at your option) any later version.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+*/
+
 #include "cup.h"
 #include "free.h"
 #include "fixture.h"
@@ -143,7 +168,8 @@ query_cup_choose_team_is_league(const gchar *sid)
     return FALSE;
 }
 
-/** Write the cup or league of the chooseteam into the appropriate pointer. */
+/** Write the cup or league of the chooseteam into the appropriate pointer
+    and return TRUE; return FALSE if no cup/league is found. */
 void
 cup_get_choose_team_league_cup(const CupChooseTeam *ct, 
 			       const League **league, const Cup **cup)
@@ -156,12 +182,12 @@ cup_get_choose_team_league_cup(const CupChooseTeam *ct,
 
     sscanf(ct->sid, "%[^0-9]%d", trash, &idx);
 
-    if(g_str_has_prefix(ct->sid, "LEAGUE"))
+    if(g_str_has_prefix(ct->sid, "LEAGUE") && idx >=0 && idx <= ligs->len)
     {
 	*league = &lig(idx - 1);
 	*cup = NULL;
     }
-    else if(g_str_has_prefix(ct->sid, "CUP"))
+    else if(g_str_has_prefix(ct->sid, "CUP") && idx >=0 && idx <= cps->len)
     {
 	*cup = &cp(idx - 1);
 	*league = NULL;
@@ -190,11 +216,9 @@ cup_get_choose_team_league_cup(const CupChooseTeam *ct,
     }
 
     if(*league == NULL && *cup == NULL)
-    {
-	g_warning("cup_get_choose_team_league_cup: no league nor cup found for chooseteam %s",
-		  ct->sid);
-	main_exit_program(EXIT_CHOOSE_TEAM_ERROR, NULL);
-    }
+	main_exit_program(EXIT_CHOOSE_TEAM_ERROR, 
+			  "cup_get_choose_team_league_cup: no league nor cup found for chooseteam %s",
+			  ct->sid);
 }
 
 /** Load the pointers to the teams participating in the 
@@ -208,7 +232,7 @@ cup_get_team_pointers(Cup *cup, gint round)
     GPtrArray *teams = cup_round->team_ptrs;
 
     if(debug > 60)
-	printf("cup_get_team_pointers %s \n", cup->name);
+	g_print("cup_get_team_pointers %s round %d\n", cup->name, round);
 
     if(teams->len > 0)
 	g_warning("cup_get_team_pointers: round %d in cup %s has non-empty team pointers array.",
@@ -216,11 +240,13 @@ cup_get_team_pointers(Cup *cup, gint round)
 
     for(i=0;i<cup_round->choose_teams->len;i++)
 	if(g_array_index(cup_round->choose_teams, CupChooseTeam, i).generate)
-	    cup_load_choose_team_generate(cup, cup_round,
-					  &g_array_index(cup_round->choose_teams, CupChooseTeam, i));
+	    cup_load_choose_team_generate(
+		cup, cup_round,
+		&g_array_index(cup_round->choose_teams, CupChooseTeam, i));
 	else
-	    cup_load_choose_team(cup, teams, 
-				 &g_array_index(cup_round->choose_teams, CupChooseTeam, i));
+	    cup_load_choose_team(
+		cup, teams, 
+		&g_array_index(cup_round->choose_teams, CupChooseTeam, i));
 
     if(cup_round->teams->len > 0)
 	while(teams->len + cup_round->teams->len > cup_round->new_teams)
@@ -231,14 +257,17 @@ cup_get_team_pointers(Cup *cup, gint round)
 
     for(i=0;i<cup_round->teams->len;i++)
     {	
-	team_generate_players_stadium(&g_array_index(cup_round->teams, Team, i));
+	team_generate_players_stadium(&g_array_index(cup_round->teams, Team, i), 0);
 	g_ptr_array_add(cup->teams, &g_array_index(cup_round->teams, Team, i));	    
 	g_ptr_array_add(teams, &g_array_index(cup_round->teams, Team, i));
     }
 
     if(debug > 70)
 	for(i=0;i<teams->len;i++)
-	    printf("cup_get_team_pointers: %d %s \n", i, ((Team*)g_ptr_array_index(teams, i))->name);
+	    g_print("cup_get_team_pointers: %d %s (%d) %s\n", i,
+		   ((Team*)g_ptr_array_index(teams, i))->name,
+		   ((Team*)g_ptr_array_index(teams, i))->clid,
+		   cup->name);
 }
 
 /** Get the pointers to the teams (already generated, in one of the leagues or cups)
@@ -254,7 +283,7 @@ cup_load_choose_team(Cup *cup, GPtrArray *teams, const CupChooseTeam *ct)
     const Cup *cup_temp = NULL;
 
     if(debug > 60)
-	printf("cup_load_choose_team: %s, %s \n", cup->name,
+	g_print("cup_load_choose_team: %s, %s \n", cup->name,
 	       ct->sid);
 
     cup_get_choose_team_league_cup(ct, &league, &cup_temp);
@@ -288,7 +317,7 @@ cup_load_choose_team(Cup *cup, GPtrArray *teams, const CupChooseTeam *ct)
 	    for(j = 0; j < end; j++)
 	    {
 		if(debug > 80)
-		    printf("team %s isinint %d numteams %d\n",
+		    g_print("team %s isinint %d numteams %d\n",
 			   team_of_id(g_array_index(league->table.elements, 
 						    TableElement, order[j]).team_id)->name,
 			   query_team_is_in_cups(
@@ -315,22 +344,20 @@ cup_load_choose_team(Cup *cup, GPtrArray *teams, const CupChooseTeam *ct)
 
 	if(ct->number_of_teams != -1 &&
 	   number_of_teams != ct->number_of_teams)
-	{
-	    g_warning("cup_load_choose_team (1): not enough teams (that don't participate in international cups yet) found in chooseteam %s for cup %s (%d specified, %d found) cup group %d.\n ",
-		      ct->sid, cup->name, ct->number_of_teams, number_of_teams, cup->group);
-	    main_exit_program(EXIT_CHOOSE_TEAM_ERROR, NULL);
-	}
+	    main_exit_program(EXIT_CHOOSE_TEAM_ERROR, 
+			      "cup_load_choose_team (1): not enough teams (that don't participate in international cups yet) found in chooseteam %s for cup %s (%d specified, %d found) cup group %d.\n ",
+			      ct->sid, cup->name, ct->number_of_teams, 
+			      number_of_teams, cup->group);
     }
     else
     {
 	if(season == 1 && cup->add_week == 0)
 	{
 	    if(lig(0).teams->len < ct->number_of_teams)
-	    {
-		g_warning("cup_load_choose_team: not enough teams in league 0 for chooseteam %s (%d; required: %d) in cup %s\n",
-			  ct->sid, lig(0).teams->len, ct->number_of_teams, cup->name);
-		main_exit_program(EXIT_CHOOSE_TEAM_ERROR, NULL);
-	    }
+		main_exit_program(EXIT_CHOOSE_TEAM_ERROR, 
+				  "cup_load_choose_team: not enough teams in league 0 for chooseteam %s (%d; required: %d) in cup %s\n",
+				  ct->sid, lig(0).teams->len, 
+				  ct->number_of_teams, cup->name);
 
 	    gint permutation[lig(0).teams->len];
 	    math_generate_permutation(permutation, 0, lig(0).teams->len - 1);
@@ -354,11 +381,10 @@ cup_load_choose_team(Cup *cup, GPtrArray *teams, const CupChooseTeam *ct)
 	    }
 	    
 	    if(number_of_teams != ct->number_of_teams)
-	    {
-		g_warning("cup_load_choose_team (2): not enough teams found in league 0 for chooseteam %s (%d; required: %d) in cup %s (group %d)\n",
-			  ct->sid, number_of_teams, ct->number_of_teams, cup->name, cup->group);
-		main_exit_program(EXIT_CHOOSE_TEAM_ERROR, NULL);
-	    }
+		main_exit_program(EXIT_CHOOSE_TEAM_ERROR, 
+				  "cup_load_choose_team (2): not enough teams found in league 0 for chooseteam %s (%d; required: %d) in cup %s (group %d)\n",
+				  ct->sid, number_of_teams, 
+				  ct->number_of_teams, cup->name, cup->group);
 	}
 	else
 	{
@@ -394,17 +420,16 @@ cup_load_choose_team(Cup *cup, GPtrArray *teams, const CupChooseTeam *ct)
 
 	    if(ct->number_of_teams != -1 &&
 	       number_of_teams != ct->number_of_teams)
-	    {
-		g_warning("cup_load_choose_team(3): not enough teams (that don't participate in international cups yet) found in chooseteam %s for cup %s (%d specified, %d found) cup group %d.\n ",
-			  ct->sid, cup->name, ct->number_of_teams, number_of_teams, cup->group);
-		main_exit_program(EXIT_CHOOSE_TEAM_ERROR, NULL);
-	    }
+		main_exit_program(EXIT_CHOOSE_TEAM_ERROR, 
+				  "cup_load_choose_team(3): not enough teams (that don't participate in international cups yet) found in chooseteam %s for cup %s (%d specified, %d found) cup group %d.\n ",
+				  ct->sid, cup->name,
+				  ct->number_of_teams, number_of_teams, cup->group);
 	}
     }
 
     if(debug > 80)
 	for(i=debug_num;i<teams->len;i++)
-	    printf("cup_load_choose_team: %d %s \n", i, ((Team*)g_ptr_array_index(teams, i))->name);
+	    g_print("cup_load_choose_team: %d %s \n", i, ((Team*)g_ptr_array_index(teams, i))->name);
 }
 
 /** Load the teams specified in the chooseteam from a non-country league. */
@@ -418,7 +443,7 @@ cup_load_choose_team_generate(Cup *cup, CupRound *cup_round, const CupChooseTeam
     GPtrArray *sids = NULL;
 
     if(debug > 60)
-	printf("cup_load_choose_team_generate: %s, %s \n", cup->name,
+	g_print("cup_load_choose_team_generate: %s, %s \n", cup->name,
 	       ct->sid);
 
     leagues = g_array_new(FALSE, FALSE, sizeof(League));
@@ -471,11 +496,11 @@ cup_load_choose_team_generate(Cup *cup, CupRound *cup_round, const CupChooseTeam
 
     if(teams_local->len < end_idx)
     {
-	g_warning("cup_load_choose_team_generate: not enough teams (%d) in chooseteam %s in cup %s (%d are specified) \n",
-		  teams_local->len, ct->sid, cup->name, end_idx);
-	
 	free_teams_array(&teams_local, FALSE);
-	main_exit_program(EXIT_CHOOSE_TEAM_ERROR, NULL);
+	main_exit_program(EXIT_CHOOSE_TEAM_ERROR, 
+			  "cup_load_choose_team_generate: not enough teams (%d) in chooseteam %s in cup %s (%d are specified) \n",
+			  teams_local->len, ct->sid, 
+			  cup->name, end_idx);
     }
 
     for(j = 0; j < end_idx; j++)
@@ -497,11 +522,11 @@ cup_load_choose_team_generate(Cup *cup, CupRound *cup_round, const CupChooseTeam
     if(((ct->number_of_teams != -1 && number_of_teams != ct->number_of_teams) ||
 	(ct->number_of_teams == -1 && number_of_teams != teams_local->len)) &&
        teams_local->len > 0)
-    {
-	g_warning("cup_load_choose_team_generate: not enough teams (that don't participate in international cups yet) found in chooseteam %s for cup %s (%d specified, %d found).\n ", ct->sid, cup->name, ct->number_of_teams, number_of_teams);
-	main_exit_program(EXIT_CHOOSE_TEAM_ERROR, NULL);
-    }
-
+	main_exit_program(EXIT_CHOOSE_TEAM_ERROR, 
+			  "cup_load_choose_team_generate: not enough teams (that don't participate in international cups yet) found in chooseteam %s for cup %s (%d specified, %d found).\n ", 
+			  ct->sid, cup->name, 
+			  ct->number_of_teams, number_of_teams);
+    
     for(j=teams_local->len - 1; j>=0;j--)
 	if(query_team_is_in_cup(&g_array_index(teams_local, Team, j), cup))
 	    g_array_remove_index(teams_local, j);
@@ -668,11 +693,10 @@ cup_get_first_week_of_cup_round(const Cup *cup, gint cup_round)
 	    cup_get_matchdays_in_cup_round(cup, cup_round) * cup->week_gap;
 
     if(week_number <= 0)
-    {
-	g_warning("cup_get_first_week_of_cup_round: first week of cup %s cup round %d is not positive (%d).\nPlease lower the week gap or set a later last week.\n", cup->name, cup_round, week_number);
-	main_exit_program(EXIT_FIRST_WEEK_ERROR, NULL);
-    }
-
+	main_exit_program(EXIT_FIRST_WEEK_ERROR, 
+			  "cup_get_first_week_of_cup_round: first week of cup %s cup round %d is not positive (%d).\nPlease lower the week gap or set a later last week.\n", 
+			  cup->name, cup_round, week_number);
+    
     return week_number;
 }
 
@@ -703,15 +727,16 @@ cup_get_matchdays_in_cup_round(const Cup *cup, gint round)
     if(cup_round->round_robin_number_of_groups > 0)
     {
 	number_of_teams = cup_round_get_number_of_teams(cup, round);
-	
-	if(number_of_teams % cup_round->round_robin_number_of_groups == 0)
-	    number_of_matchdays = 
-		((number_of_teams / cup_round->round_robin_number_of_groups) - 1) * 2;
+	number_of_teams += cup_round->round_robin_number_of_groups - 1;
+	number_of_teams /= cup_round->round_robin_number_of_groups;
+	/* Now, number_of_teams = number of teams in largest group */
+	if (number_of_teams % 2 == 0)
+	    number_of_matchdays = number_of_teams - 1;
 	else
-	    number_of_matchdays =
-		((number_of_teams - 
-		  (number_of_teams % cup_round->round_robin_number_of_groups)) /
-		 cup_round->round_robin_number_of_groups) * 2;
+	    number_of_matchdays = number_of_teams;
+	
+	if (g_array_index(cup->rounds, CupRound, round).home_away)
+	    number_of_matchdays *= 2;
     }
     else if(g_array_index(cup->rounds, CupRound, round).home_away)
 	number_of_matchdays = 2;
@@ -836,9 +861,8 @@ cup_from_clid(gint clid)
 	if(cp(i).id == clid)
 	    return &cp(i);
 
-    g_warning("cup_from_clid: didn't find cup with id %d\n", clid);
-
-    main_exit_program(EXIT_POINTER_NOT_FOUND, NULL);
+    main_exit_program(EXIT_POINTER_NOT_FOUND, 
+		      "cup_from_clid: didn't find cup with id %d\n", clid);
 
     return NULL;
 }
@@ -853,9 +877,8 @@ cup_from_sid(const gchar *sid)
 	if(strcmp(cp(i).sid, sid) == 0)
 	    return &cp(i);
 
-    g_warning("cup_from_sid: didn't find cup with sid %s \n", sid);
-
-    main_exit_program(EXIT_POINTER_NOT_FOUND, NULL);
+    main_exit_program(EXIT_POINTER_NOT_FOUND, 
+		      "cup_from_sid: didn't find cup with sid %s \n", sid);
 
     return NULL;
 }
@@ -949,55 +972,34 @@ cup_get_winner(const Cup *cup)
 gboolean
 query_cup_begins(const Cup *cup)
 {
-    gint i;
+    gint i, j;
     const League *league = NULL;
     const Cup *cup_temp = NULL;
-    gboolean proceed = FALSE;
-    const CupRound *cup_round = &g_array_index(cup->rounds, CupRound, 0);
+    const CupRound *cup_round = NULL;
 
-    for(i=0;i<cup_round->choose_teams->len;i++)
-	if(!g_array_index(cup_round->choose_teams,CupChooseTeam, i).generate)
-	{
-	    
-	    cup_get_choose_team_league_cup(
-		&g_array_index(cup_round->choose_teams,
-			       CupChooseTeam, i), &league, &cup_temp);
+    for(j=0;j<cup->rounds->len;j++)
+    {
+	cup_round = &g_array_index(cup->rounds, CupRound, j);
 
-	    if((cup_temp == NULL && 
-		(!league->active || 
-		 (g_array_index(league->fixtures, Fixture,
-				league->fixtures->len - 1).week_number == week &&
-		  g_array_index(league->fixtures, Fixture,
-				league->fixtures->len - 1).week_round_number == week_round))) ||
-	       (league == NULL && 
-		(cup_temp->fixtures->len > 0 &&
-		 g_array_index(cup_temp->fixtures, Fixture,
-			       cup_temp->fixtures->len - 1).week_number == week &&
-		 g_array_index(cup_temp->fixtures, Fixture,
-			       cup_temp->fixtures->len - 1).week_round_number == week_round)))
-		proceed = TRUE;
-	}
-    
-    if(!proceed)
-	return FALSE;
-
-    for(i=0;i<cup_round->choose_teams->len;i++)
-	if(!g_array_index(cup_round->choose_teams,CupChooseTeam, i).generate)
-	{
-	    cup_get_choose_team_league_cup(
-		&g_array_index(cup_round->choose_teams,
-			       CupChooseTeam, i), &league, &cup_temp);
-	    if((cup_temp == NULL &&
-		(league->active &&
-		 g_array_index(league->fixtures, Fixture,
-			       league->fixtures->len - 1).attendance == -1)) ||
-	       (league == NULL &&
-		(cup_temp->fixtures->len == 0 || 
-		 (cup_temp->fixtures->len > 0 && 
-		  g_array_index(cup_temp->fixtures, Fixture,
-				cup_temp->fixtures->len - 1).attendance == -1))))
-		return FALSE;
-	}
+	for(i=0;i<cup_round->choose_teams->len;i++)
+	    if(!g_array_index(cup_round->choose_teams,CupChooseTeam, i).generate)
+	    {		
+		cup_get_choose_team_league_cup(
+		    &g_array_index(cup_round->choose_teams,
+				   CupChooseTeam, i), &league, &cup_temp);
+		
+		if((cup_temp == NULL &&
+		    league->active &&
+		    g_array_index(league->fixtures, Fixture,
+				  league->fixtures->len - 1).attendance == -1) ||
+		   (league == NULL &&
+		    ((cup_temp->fixtures->len > 0 &&
+		      g_array_index(cup_temp->fixtures, Fixture,
+				    cup_temp->fixtures->len - 1).attendance == -1) ||
+		     cup_temp->fixtures->len == 0)))
+		    return FALSE;
+	    }
+    }
 
     return TRUE;
 }
