@@ -1,3 +1,28 @@
+/*
+   fixture.c
+
+   Bygfoot Football Manager -- a small and simple GTK2-based
+   football management game.
+
+   http://bygfoot.sourceforge.net
+
+   Copyright (C) 2005  Gyözö Both (gyboth@bygfoot.com)
+
+   This program is free software; you can redistribute it and/or
+   modify it under the terms of the GNU General Public License
+   as published by the Free Software Foundation; either version 2
+   of the License, or (at your option) any later version.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+*/
+
 #include "cup.h"
 #include "fixture.h"
 #include "free.h"
@@ -46,13 +71,13 @@ fixture_write_cup_fixtures(Cup *cup)
 	cup_get_team_pointers(cup, i);
 
     if(g_array_index(cup->rounds, CupRound, 0).round_robin_number_of_groups > 0)
-	fixture_write_cup_round_robin(cup, 0, 
-				      misc_copy_ptr_array(
-					  g_array_index(cup->rounds, CupRound, 0).team_ptrs));
+	fixture_write_cup_round_robin(
+	    cup, 0, misc_copy_ptr_array(
+		g_array_index(cup->rounds, CupRound, 0).team_ptrs));
     else
-	fixture_write_knockout_round(cup, 0, 
-				     misc_copy_ptr_array(
-					 g_array_index(cup->rounds, CupRound, 0).team_ptrs));
+	fixture_write_knockout_round(
+	    cup, 0, misc_copy_ptr_array(
+		g_array_index(cup->rounds, CupRound, 0).team_ptrs));
 }
 
 /** Update the fixtures for the given cup. 
@@ -67,6 +92,9 @@ fixture_update(Cup *cup)
     GPtrArray *teams = NULL;
     const CupRound *new_round = NULL;
 
+    if(debug > 90)
+	g_print("fixture_update: cup %s (id %d)\n", cup->name, cup->id);
+
     if(replay != 0 && 
        g_array_index(fixtures, Fixture, fixtures->len - 1).replay_number < replay && 
        fixture_update_write_replays(cup))
@@ -82,10 +110,11 @@ fixture_update(Cup *cup)
 
     if(round + 1 > cup->rounds->len - 1)
     {
-	g_warning("fixture_update: round index %d too high for round array (%d) in cup %s\n",
-		  round + 1, cup->rounds->len - 1, cup->name);
 	g_ptr_array_free(teams, TRUE);
-	main_exit_program(EXIT_CUP_ROUND_ERROR, NULL);
+	main_exit_program(
+	    EXIT_CUP_ROUND_ERROR, 
+	    "fixture_update: round index %d too high for round array (%d) in cup %s\n",
+	    round + 1, cup->rounds->len - 1, cup->name);
     }
 
     new_round = &g_array_index(cup->rounds, CupRound, round + 1);
@@ -286,12 +315,11 @@ fixture_write_cup_round_robin(Cup *cup, gint cup_round, GPtrArray *teams)
     TableElement new_table_element;
 
     if(teams->len < number_of_groups)
-    {
-	g_warning("fixture_write_cup_round_robin: cup %s round %d: number of teams (%d) less than number of groups (%d)\n", cup->name, cup_round, teams->len, number_of_groups);
-	main_exit_program(EXIT_FIXTURE_WRITE_ERROR, NULL);
-    }
+	main_exit_program(EXIT_FIXTURE_WRITE_ERROR, 
+			  "fixture_write_cup_round_robin: cup %s round %d: number of teams (%d) less than number of groups (%d)\n", 
+			  cup->name, cup_round, teams->len, number_of_groups);
 
-    if(cupround->randomise_teams)
+    if(cupround->randomise_teams || opt_int("int_opt_randomise_teams"))
 	g_ptr_array_sort_with_data(teams, (GCompareDataFunc)team_compare_func,
 			       GINT_TO_POINTER(TEAM_COMPARE_AV_SKILL));
 
@@ -306,7 +334,7 @@ fixture_write_cup_round_robin(Cup *cup, gint cup_round, GPtrArray *teams)
     }
     
     cnt = 0;
-    if(!cupround->randomise_teams)
+    if(!cupround->randomise_teams && !opt_int("int_opt_randomise_teams"))
     {
 	for(i=0;i<number_of_groups;i++)
 	    for(j=0;j<team_num + (i < plus_div);j++)
@@ -398,11 +426,10 @@ fixture_write_round_robin(gpointer league_cup, gint cup_round, GPtrArray *teams,
     first_fixture = fixtures->len;
 
     if(first_week < 1)
-    {	
-	g_warning("fixture_write_round_robin: first week of %s is not positive (%d).\nPlease lower the week gap or set a later last week.\n", league_cup_get_name_string(clid), first_week);
-	main_exit_program(EXIT_FIXTURE_WRITE_ERROR, NULL);
-    }
-
+	main_exit_program(EXIT_FIXTURE_WRITE_ERROR, 
+			  "fixture_write_round_robin: first week of %s is not positive (%d).\nPlease lower the week gap or set a later last week.\n", 
+			  league_cup_get_name_string(clid), first_week);
+    
     if(len % 2 != 0)
     {
 	team_temp = team_new(FALSE);
@@ -503,6 +530,10 @@ fixture_write_knockout_round(Cup *cup, gint cup_round, GPtrArray *teams)
     gint bye_len = (round->byes == -1) ?
 	math_get_bye_len(len) : round->byes;
 
+    if(debug > 60)
+	g_print("fixture_write_knockout_round: %s %d byelen %d\n",
+	       cup->name, cup_round, bye_len);
+
     if(bye_len != 0)
     {
 	cup->bye = g_ptr_array_new();
@@ -535,7 +566,8 @@ fixture_write_knockout_round(Cup *cup, gint cup_round, GPtrArray *teams)
 	    fixture_get_free_round(first_week + cup->week_gap, teams, -1, -1);
 	for(i=0; i<=(teams->len - 2) / 2; i++)
 	    fixture_write(cup->fixtures, (Team*)g_ptr_array_index(teams, 2 * i + 1),
-			  (Team*)g_ptr_array_index(teams, 2 * i), first_week + cup->week_gap,
+			  (Team*)g_ptr_array_index(teams, 2 * i), 
+			  first_week + cup->week_gap,
 			  week_round_number, cup->id, cup_round, 0,
 			  !round->neutral, TRUE, TRUE);
     }
@@ -543,11 +575,14 @@ fixture_write_knockout_round(Cup *cup, gint cup_round, GPtrArray *teams)
     g_array_sort_with_data(cup->fixtures, fixture_compare_func,
 			   GINT_TO_POINTER(FIXTURE_COMPARE_DATE + 100));
 
-    cup->next_fixture_update_week = (cup_round < cup->rounds->len - 1 || round->replay > 0) ?
+    cup->next_fixture_update_week = 
+	(cup_round < cup->rounds->len - 1 || round->replay > 0) ?
 	g_array_index(cup->fixtures, Fixture, cup->fixtures->len - 1).week_number : -1;
-    cup->next_fixture_update_week_round = (cup_round < cup->rounds->len - 1 || round->replay > 0) ?
+    
+    cup->next_fixture_update_week_round = 
+	(cup_round < cup->rounds->len - 1 || round->replay > 0) ?
 	g_array_index(cup->fixtures, Fixture, cup->fixtures->len - 1).week_round_number : -1;
-
+    
     g_ptr_array_free(teams, TRUE);
 }
 
@@ -1083,7 +1118,7 @@ fixture_get_matches(const Team *tm1, const Team *tm2)
 
 /** Return the fixture going with the id. */
 Fixture*
-fixture_from_id(gint id)
+fixture_from_id(gint id, gboolean abort_program)
 {
     gint i, j;
 
@@ -1097,9 +1132,9 @@ fixture_from_id(gint id)
 	    if(g_array_index(cp(i).fixtures, Fixture, j).id == id)
 		return &g_array_index(cp(i).fixtures, Fixture, j);
 
-    g_warning("fixture_from_id: fixture with id %d found \n", id);
-
-    main_exit_program(EXIT_POINTER_NOT_FOUND, NULL);
+    if(abort_program)
+	main_exit_program(EXIT_POINTER_NOT_FOUND, 
+			  "fixture_from_id: fixture with id %d found \n", id);
 
     return NULL;
 }
@@ -1249,3 +1284,69 @@ fixture_get_season_results(void)
 
     return results;
 }
+
+/** Calculate how many goals the team has to score to
+    win the match (also taking into account second leg stuff). */
+gint
+fixture_get_goals_to_win(const Fixture *fix, const Team *tm)
+{
+    const Fixture *first_leg = NULL;
+    gint idx = (tm == fix->teams[1]), return_value = -100;
+    gint res[2] = {math_sum_int_array(fix->result[0], 2),
+		   math_sum_int_array(fix->result[1], 2)};
+    gint tmp;
+
+    if(!query_fixture_team_involved(fix, tm->id))
+    {
+	g_warning("fixture_get_goals_to_win: team %s doesn't participate in fixture given (%s)\n",
+		  tm->name, league_cup_get_name_string(fix->clid));
+	return -100;
+    }
+
+    if(!fix->second_leg)
+	return res[!idx] - res[idx] + 1;
+
+    first_leg = fixture_get_first_leg(fix);
+
+    if(res[0] + first_leg->result[1][0] ==
+       res[1] + first_leg->result[0][0])
+    {
+	if(idx == 0)
+	    return_value = (res[1] >= first_leg->result[1][0]) ?
+		1 : 0;
+	else
+	    return_value = (res[1] > first_leg->result[1][0]) ?
+		0 : 1;
+    }
+    else if(res[0] + first_leg->result[1][0] >
+       res[1] + first_leg->result[0][0])
+    {
+	tmp = res[1];
+	while(res[0] + first_leg->result[1][0] >
+	      res[1] + first_leg->result[0][0])
+	    res[1]++;
+
+	if(idx == 0)
+	    return_value = (res[1] >= first_leg->result[1][0]) ?
+		tmp - res[1] + 1 : tmp - res[1];
+	else
+	    return_value = (res[1] > first_leg->result[1][0]) ?
+		res[1] - tmp : res[1] - tmp + 1;
+    }
+    else
+    {
+	tmp = res[0];
+	while(res[0] + first_leg->result[1][0] <
+	      res[1] + first_leg->result[0][0])
+	    res[0]++;
+
+	if(idx == 0)
+	    return_value = (res[1] >= first_leg->result[1][0]) ?
+		res[0] - tmp + 1 : res[0] - tmp;
+	else
+	    return_value = (res[1] > first_leg->result[1][0]) ?
+		tmp - res[0] : tmp - res[0] + 1;
+    }
+
+    return return_value;
+}
