@@ -1,3 +1,28 @@
+/*
+   game.c
+
+   Bygfoot Football Manager -- a small and simple GTK2-based
+   football management game.
+
+   http://bygfoot.sourceforge.net
+
+   Copyright (C) 2005  Gyözö Both (gyboth@bygfoot.com)
+
+   This program is free software; you can redistribute it and/or
+   modify it under the terms of the GNU General Public License
+   as published by the Free Software Foundation; either version 2
+   of the License, or (at your option) any later version.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+*/
+
 #include "cup.h"
 #include "finance.h"
 #include "fixture.h"
@@ -39,24 +64,26 @@ game_get_values(const Fixture *fix, gfloat team_values[][GAME_TEAM_VALUE_END],
 	style_factor = (gfloat)tm[i]->style * const_float("float_game_style_factor");
 
 	team_values[i][GAME_TEAM_VALUE_GOALIE] = 
-	    game_get_player_contribution(player_of_idx_team(tm[i], 0), FALSE) * 
-	    (1 + home_advantage * (i == 0)) *
-	    (1 + const_float("float_player_boost_skill_effect") * tm[i]->boost);
+	    game_get_player_contribution(player_of_idx_team(tm[i], 0), FALSE, TRUE) * 
+	    (1 + home_advantage * (i == 0));
 
 	for(j=1;j<11;j++)
-	{
-	    team_values[i][GAME_TEAM_VALUE_ATTACK] +=
-		game_get_player_contribution(player_of_idx_team(tm[i], j), GAME_TEAM_VALUE_ATTACK);
-	    team_values[i][GAME_TEAM_VALUE_MIDFIELD] +=
-		game_get_player_contribution(player_of_idx_team(tm[i], j), GAME_TEAM_VALUE_MIDFIELD);
-	    team_values[i][GAME_TEAM_VALUE_DEFEND] +=
-		game_get_player_contribution(player_of_idx_team(tm[i], j), GAME_TEAM_VALUE_DEFEND);
-	}
+	    if(player_of_idx_team(tm[i], j)->cskill > 0)
+	    {
+		team_values[i][GAME_TEAM_VALUE_ATTACK] +=
+		    game_get_player_contribution(player_of_idx_team(tm[i], j), 
+						 GAME_TEAM_VALUE_ATTACK, TRUE);
+		team_values[i][GAME_TEAM_VALUE_MIDFIELD] +=
+		    game_get_player_contribution(player_of_idx_team(tm[i], j), 
+						 GAME_TEAM_VALUE_MIDFIELD, TRUE);
+		team_values[i][GAME_TEAM_VALUE_DEFEND] +=
+		    game_get_player_contribution(player_of_idx_team(tm[i], j), 
+						 GAME_TEAM_VALUE_DEFEND, TRUE);
+	    }
 
 	for(j=GAME_TEAM_VALUE_DEFEND;j<GAME_TEAM_VALUE_DEFEND + 3;j++)
 	    team_values[i][j] *= 
-		((1 + home_advantage * (i == 0)) *
-		 (1 + const_float("float_player_boost_skill_effect") * tm[i]->boost));
+		((1 + home_advantage * (i == 0)));
 
 	team_values[i][GAME_TEAM_VALUE_DEFEND] *= (1 - style_factor);
 	team_values[i][GAME_TEAM_VALUE_ATTACK] *= (1 + style_factor);
@@ -66,10 +93,12 @@ game_get_values(const Fixture *fix, gfloat team_values[][GAME_TEAM_VALUE_END],
 /** Return the contribution of a player to the attack, midfield or defend.
     @param pl The player.
     @param type Whether we have defend, midfield or attack value.
+    @param special Whether to count special influence like boost and
+    streak.
     @return The player's contribution depending on position and
     fitness. */
 gfloat
-game_get_player_contribution(const Player *pl, gint type)
+game_get_player_contribution(const Player *pl, gint type, gboolean special)
 {
 /** How the cskill of field players get weighted for the team values in
     a match. Rows are player position, columns value type. 
@@ -85,9 +114,8 @@ game_get_player_contribution(const Player *pl, gint type)
 	  const_float("float_player_team_weight_forward_midfield"), 
 	  const_float("float_player_team_weight_forward_attack")}};
 
-    return player_get_game_skill(pl, FALSE) *
-	player_weights[pl->cpos - 1][type - GAME_TEAM_VALUE_DEFEND] *
-	(1 + (gfloat)pl->streak * const_float("float_player_streak_influence_skill"));
+    return player_get_game_skill(pl, FALSE, special) *
+	player_weights[pl->cpos - 1][type - GAME_TEAM_VALUE_DEFEND];
 }
 
 /** Return a random attacking or defending player
@@ -130,10 +158,8 @@ game_get_player(const Team *tm, gint player_type,
     else if(player_type == GAME_PLAYER_TYPE_PENALTY)
 	return game_get_penalty_taker(tm, last_penalty);
     else
-    {
-	g_warning("game_get_player: unknown player type %d\n", player_type);
-	main_exit_program(EXIT_INT_NOT_FOUND, NULL);
-    }
+	main_exit_program(EXIT_INT_NOT_FOUND, 
+			  "game_get_player: unknown player type %d\n", player_type);
 
     game_get_player_probs(tm->players, probs, weights, skills);
 
@@ -154,12 +180,12 @@ game_get_player(const Team *tm, gint player_type,
     else
     {
 	g_warning("game_get_player: All players injured or banned, apparently.\n");
-	printf("%s %s player list:\n", league_cup_get_name_string(tm->clid), tm->name);
+	g_print("%s %s player list:\n", league_cup_get_name_string(tm->clid), tm->name);
 	for(i=0;i<tm->players->len;i++)
 	{
 	    if(i < 10)
-		printf("prob %.3f  ", probs[i]);
-	    printf("%d %20s health %d cskill %.2f\n", i, player_of_idx_team(tm, i)->name,
+		g_print("prob %.3f  ", probs[i]);
+	    g_print("%d %20s health %d cskill %.2f\n", i, player_of_idx_team(tm, i)->name,
 		   player_of_idx_team(tm, i)->health, player_of_idx_team(tm, i)->cskill);
 	}
 
@@ -183,7 +209,7 @@ game_get_player_probs(GArray *players, gfloat *probs, gfloat *weights, gboolean
     gint i;
 
     probs[0] = (skills) ? 
-	player_get_game_skill(&g_array_index(players, Player, 1), FALSE) *
+	player_get_game_skill(&g_array_index(players, Player, 1), FALSE, TRUE) *
 	weights[g_array_index(players, Player, 1).cpos - 1] :
 	weights[g_array_index(players, Player, 1).cpos - 1] *
 	(g_array_index(players, Player, 1).cskill != 0);
@@ -196,7 +222,7 @@ game_get_player_probs(GArray *players, gfloat *probs, gfloat *weights, gboolean
     {
 	probs[i] = probs[i - 1] + 
 	    ((skills) ? 
-	     player_get_game_skill(&g_array_index(players, Player, i + 1), FALSE) *
+	     player_get_game_skill(&g_array_index(players, Player, i + 1), FALSE, TRUE) *
 	     weights[g_array_index(players, Player, i + 1).cpos - 1] :
 	     weights[g_array_index(players, Player, i + 1).cpos - 1] *
 	     (g_array_index(players, Player, i + 1).cskill != 0));
@@ -388,7 +414,8 @@ game_assign_attendance_neutral(Fixture *fix)
 {
     const GPtrArray *teamsp = 
 	(GPtrArray*)league_cup_get_teams(fix->clid);
-    gfloat av_att = (query_cup_is_international(fix->clid) && teamsp->len > 0) ?
+    gfloat av_att = (fix->clid >= ID_CUP_START && 
+		     query_cup_is_international(fix->clid) && teamsp->len > 0) ?
 	(gfloat)league_cup_average_capacity(fix->clid) :
 	(gfloat)league_cup_average_capacity(lig(0).id);
 
@@ -643,6 +670,13 @@ game_substitute_player(Team *tm, gint player_number)
 	if(g_array_index(tm->players, Player, i).cskill > 0)
 	    g_ptr_array_add(substitutes, &g_array_index(tm->players, Player, i));
 
+    if(substitutes->len == 0)
+    {
+	g_ptr_array_free(substitutes, TRUE);
+	g_warning("game_substitute_player: no suitable substitutes found (all injured/banned?)");
+	return -1;
+    }
+
     g_ptr_array_sort_with_data(substitutes, 
 			       (GCompareDataFunc)player_compare_substitute_func,
 			       GINT_TO_POINTER(player_of_idx_team(tm, 
@@ -659,13 +693,6 @@ game_substitute_player(Team *tm, gint player_number)
 
     substitute = ((Player*)g_ptr_array_index(substitutes, 0))->id;
 
-    player_streak_add_to_prob(
-	&g_array_index(tm->players, Player, player_number),
-	const_float("float_player_streak_add_sub_out"));
-
-    player_streak_add_to_prob((Player*)g_ptr_array_index(substitutes, 0),
-			      const_float("float_player_streak_add_sub_in"));
-
     player_swap(tm, player_number, tm, player_id_index(tm, substitute));
 
     g_ptr_array_free(substitutes, TRUE);
@@ -787,6 +814,14 @@ game_substitute_player_send_off(gint clid, Team *tm, gint player_number,
 	if(g_array_index(tm->players, Player, i).cskill > 0)
 	    g_ptr_array_add(substitutes, player_of_idx_team(tm, i));
 
+    if(substitutes->len == 0)
+    {
+	g_ptr_array_free(substitutes, TRUE);
+	g_warning("game_substitute_player_send_off: no suitable substitutes found (all injured/banned?)");
+	*to_substitute = -1;
+	return;
+    }
+	
     if(num_forw == 0 && MAX(num_def, num_mid) > 2)
 	position = PLAYER_POS_FORWARD;
     else
@@ -822,9 +857,10 @@ game_decrease_fitness(const Fixture *fix)
 
     for(i=0;i<2;i++)
     {
-	for(j=0;j<11;j++)
-	    if(player_of_idx_team(fix->teams[i], j)->cskill > 0)
-		player_decrease_fitness(player_of_idx_team(fix->teams[i], j));
+	if(debug < 50 || team_is_user(fix->teams[i]) == -1)
+	    for(j=0;j<11;j++)
+		if(player_of_idx_team(fix->teams[i], j)->cskill > 0)
+		    player_decrease_fitness(player_of_idx_team(fix->teams[i], j));
     }
 }
 
@@ -832,11 +868,10 @@ game_decrease_fitness(const Fixture *fix)
     @param live_game_stats Pointer to the live game.
     @param live_game_unit The live game unit. */
 void
-game_update_stats(gpointer live_game, gconstpointer live_game_unit)
+game_update_stats(LiveGame *lg, const LiveGameUnit *unit)
 {
     gint i;
-    LiveGameStats *stats = &((LiveGame*)live_game)->stats;
-    const LiveGameUnit *unit = (const LiveGameUnit*)live_game_unit;
+    LiveGameStats *stats = &lg->stats;
     
     if(unit->minute != -1)
 	stats->values[unit->possession][LIVE_GAME_STAT_VALUE_POSSESSION]++;
@@ -850,7 +885,7 @@ game_update_stats(gpointer live_game, gconstpointer live_game_unit)
     else if(unit->event.type == LIVE_GAME_EVENT_INJURY)
     {
 	stats->values[unit->event.team][LIVE_GAME_STAT_VALUE_INJURIES]++;
-	game_update_stats_player(live_game, live_game_unit);
+	game_update_stats_player(lg, unit);
     }
     else if(unit->event.type == LIVE_GAME_EVENT_FOUL ||
 	    unit->event.type == LIVE_GAME_EVENT_FOUL_YELLOW ||
@@ -861,13 +896,13 @@ game_update_stats(gpointer live_game, gconstpointer live_game_unit)
 	if(unit->event.type == LIVE_GAME_EVENT_FOUL_YELLOW)
 	{
 	    stats->values[unit->event.team][LIVE_GAME_STAT_VALUE_CARDS]++;
-	    game_update_stats_player(live_game, live_game_unit);
+	    game_update_stats_player(lg, unit);
 	}
     }
     else if(unit->event.type == LIVE_GAME_EVENT_SEND_OFF)
     {
 	stats->values[unit->event.team][LIVE_GAME_STAT_VALUE_REDS]++;
-	game_update_stats_player(live_game, live_game_unit);
+	game_update_stats_player(lg, unit);
     }
     else if(unit->event.type == LIVE_GAME_EVENT_GOAL ||
 	    unit->event.type == LIVE_GAME_EVENT_OWN_GOAL)
@@ -875,7 +910,7 @@ game_update_stats(gpointer live_game, gconstpointer live_game_unit)
 	if(live_game_unit_before(unit, -1)->event.type != LIVE_GAME_EVENT_PENALTY &&
 	   unit->event.type != LIVE_GAME_EVENT_OWN_GOAL)
 	    stats->values[unit->event.team][LIVE_GAME_STAT_VALUE_GOALS_REGULAR]++;
-	game_update_stats_player(live_game, live_game_unit);
+	game_update_stats_player(lg, unit);
     }
 
     for(i=0;i<2;i++)
@@ -891,19 +926,18 @@ game_update_stats(gpointer live_game, gconstpointer live_game_unit)
     @param player The player id.
     @param type The type of the stat. */
 void
-game_update_stats_player(gpointer live_game, gconstpointer live_game_unit)
+game_update_stats_player(LiveGame *lg, const LiveGameUnit *unit)
 {
     gint i;
     gchar buf[SMALL], buf2[SMALL];    
-    LiveGameStats *stats = &((LiveGame*)live_game)->stats;
-    const LiveGameUnit *unit = (const LiveGameUnit*)live_game_unit;
+    LiveGameStats *stats = &lg->stats;
     gint minute = live_game_unit_get_minute(unit), array_index = -1;
     gboolean own_goal;
     gint team = unit->event.team,
 	player = unit->event.player,
 	player2 = unit->event.player2;
-    const Team *tm[2] = {((LiveGame*)live_game)->fix->teams[0], 
-			 ((LiveGame*)live_game)->fix->teams[1]};
+    const Team *tm[2] = {lg->fix->teams[0], 
+			 lg->fix->teams[1]};
     GPtrArray *players = NULL;
     
     if(unit->event.type == LIVE_GAME_EVENT_GOAL ||
@@ -975,10 +1009,11 @@ game_post_match(Fixture *fix)
     gint i;
     GPtrArray *teams = NULL;
     Cup *cup = NULL;
+    gchar buf[SMALL], buf2[SMALL];
 
     if((debug > 100 && fixture_user_team_involved(fix) != -1) ||
        debug > 130)
-	printf("game_post_match: %s - %s\n", 
+	g_print("game_post_match: %s - %s\n", 
 	       fix->teams[0]->name,
 	       fix->teams[1]->name);
 
@@ -986,13 +1021,7 @@ game_post_match(Fixture *fix)
 	table_update(fix);
     
     for(i=0;i<2;i++)
-    {
-	if(team_is_user(fix->teams[i]) == -1)
-	    team_update_cpu_team(fix->teams[i],
-				 (fixture_user_team_involved(fix) != -1));
-	else
-	    team_update_post_match(fix->teams[i], fix);
-    }
+	team_update_post_match(fix->teams[i], fix);
 
     if(fix->clid < ID_CUP_START)
 	return;
@@ -1005,19 +1034,40 @@ game_post_match(Fixture *fix)
 	teams = cup_get_teams_sorted(cup);
 	
 	if(team_is_user((Team*)g_ptr_array_index(teams, 0)) != -1)
+	{
 	    user_history_add(&usr(team_is_user((Team*)g_ptr_array_index(teams, 0))),
-			     USER_HISTORY_WIN_FINAL, ((Team*)g_ptr_array_index(teams, 0))->id,
-			     fix->clid, fix->round,((Team*)g_ptr_array_index(teams, 1))->name);
+			     USER_HISTORY_WIN_FINAL, 
+			     ((Team*)g_ptr_array_index(teams, 0))->name,
+			     league_cup_get_name_string(fix->clid),
+			     ((Team*)g_ptr_array_index(teams, 1))->name, NULL);
+	    user_add_cup_success(&usr(team_is_user((Team*)g_ptr_array_index(teams, 0))),
+				 cup, fix->round, USER_HISTORY_WIN_FINAL);
+	}
 	else if(team_is_user((Team*)g_ptr_array_index(teams, 1)) != -1)
+	{
 	    user_history_add(&usr(team_is_user((Team*)g_ptr_array_index(teams, 1))),
-			     USER_HISTORY_LOSE_FINAL, ((Team*)g_ptr_array_index(teams, 1))->id,
-			     fix->clid, fix->round,((Team*)g_ptr_array_index(teams, 0))->name);
+			     USER_HISTORY_LOSE_FINAL, 
+			     ((Team*)g_ptr_array_index(teams, 1))->name,
+			     league_cup_get_name_string(fix->clid),
+			     ((Team*)g_ptr_array_index(teams, 0))->name, NULL);
+	    user_add_cup_success(&usr(team_is_user((Team*)g_ptr_array_index(teams, 1))),
+				 cup, fix->round, USER_HISTORY_LOSE_FINAL);
+	}
 	g_ptr_array_free(teams, TRUE);
     }
     else if(fixture_user_team_involved(fix) != -1)
-	user_history_add(&usr(fixture_user_team_involved(fix)), USER_HISTORY_REACH_CUP_ROUND,
-			 usr(fixture_user_team_involved(fix)).team_id,
-			 fix->clid, fix->round, "");
+    {
+	cup_get_round_name(cup_from_clid(fix->clid), fix->round, buf);
+	sprintf(buf2, "%d", fix->round + 1);
+
+	user_history_add(&usr(fixture_user_team_involved(fix)),
+			 USER_HISTORY_REACH_CUP_ROUND,
+			 usr(fixture_user_team_involved(fix)).tm->name,
+			 league_cup_get_name_string(fix->clid),
+			 buf, buf2);
+	user_add_cup_success(&usr(fixture_user_team_involved(fix)),
+			     cup, fix->round, USER_HISTORY_REACH_CUP_ROUND);
+    }
 }
 
 /** Reduce stadium capacity and safety after a stadium event.
@@ -1064,7 +1114,7 @@ game_get_max_values(gfloat max_values[4])
 	for(j=1;j<11;j++)
 	{
 	    pl.cpos = player_get_position_from_structure(442, j);
-	    max_values[i] += game_get_player_contribution(&pl, i);
+	    max_values[i] += game_get_player_contribution(&pl, i, FALSE);
 	}
     }
 }
@@ -1094,3 +1144,15 @@ game_get_default_penalty_shooter(const Team *tm)
 
     return return_value;
 }
+
+/** Deduce some money for boost during a match. */
+void
+game_boost_cost(void)
+{
+    gfloat wage_unit = finance_wage_unit(usr(stat2).tm);
+    gint deduce = 
+	(gint)rint(wage_unit * const_float("float_boost_cost_factor"));
+
+    usr(stat2).money -= deduce;
+    usr(stat2).money_out[1][MON_OUT_BOOST] -= deduce;
+}
