@@ -1,7 +1,33 @@
+/*
+   game_gui.c
+
+   Bygfoot Football Manager -- a small and simple GTK2-based
+   football management game.
+
+   http://bygfoot.sourceforge.net
+
+   Copyright (C) 2005  Gyözö Both (gyboth@bygfoot.com)
+
+   This program is free software; you can redistribute it and/or
+   modify it under the terms of the GNU General Public License
+   as published by the Free Software Foundation; either version 2
+   of the License, or (at your option) any later version.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+*/
+
 #include "callbacks.h"
 #include "file.h"
 #include "game_gui.h"
 #include "gui.h"
+#include "job.h"
 #include "league.h"
 #include "live_game.h"
 #include "maths.h"
@@ -153,6 +179,37 @@ game_gui_live_game_set_hscale(const LiveGameUnit *unit, GtkHScale *hscale)
     gtk_widget_modify_bg(GTK_WIDGET(hscale), GTK_STATE_NORMAL, &color);
 }
 
+/** Show the player list of the opposing team in the live game
+    window. */
+void
+game_gui_live_game_show_opponent(void)
+{
+    GtkImage *image_style = 
+	GTK_IMAGE(lookup_widget(window.live, "image_lg_opp_style")),
+	*image_boost = 
+	GTK_IMAGE(lookup_widget(window.live, "image_lg_opp_boost"));
+    GtkLabel *label_form = 
+	GTK_LABEL(lookup_widget(window.live, "label_lg_formation")),
+	*label_avskill = 
+	GTK_LABEL(lookup_widget(window.live, "label_lg_avskill"));
+    GtkTreeView *treeview = 
+	GTK_TREE_VIEW(lookup_widget(window.live, "treeview_lg_opponent"));
+    gint idx = (team_is_user(((LiveGame*)statp)->fix->teams[0]) == -1);
+    const Team *tm = ((LiveGame*)statp)->fix->teams[!idx];
+    gint scout = user_from_team(((LiveGame*)statp)->fix->teams[idx])->scout;
+    gfloat avskills[2] = {team_get_average_skill(tm, TRUE),
+			  team_get_average_skill(current_user.tm, TRUE)};
+    gchar buf[SMALL];
+
+    treeview_show_player_list_team(treeview, tm, scout);
+    game_gui_write_meter_images(tm, image_style, image_boost);    
+    gui_label_set_text_from_int(label_form, tm->structure, FALSE);
+
+    sprintf(buf, "%.1f (%+.1f)", avskills[0], 
+	    avskills[0] - avskills[1]);
+    gtk_label_set_text(label_avskill, buf);
+}
+
 /** Look up the widgets in the main window. */
 void
 game_gui_get_radio_items(GtkWidget **style, GtkWidget **scout,
@@ -266,13 +323,9 @@ game_gui_write_av_skills(void)
 /** Set the images for the style and boost meters to the appropriate values
     from the team settings. */
 void
-game_gui_write_meters(const Team *tm)
+game_gui_write_meter_images(const Team *tm, GtkImage *style, GtkImage *boost)
 {
     gint i;
-    GtkImage *image_style_main = GTK_IMAGE(lookup_widget(window.main, "image_style")),
-	*image_boost_main = GTK_IMAGE(lookup_widget(window.main, "image_boost"));
-    GtkImage *image_style_live = NULL,
-	*image_boost_live = NULL;
 
     gchar *image_style_files[5] = 
 	{file_find_support_file(const_app("string_game_gui_style_all_out_defend_icon"), TRUE),
@@ -285,22 +338,34 @@ game_gui_write_meters(const Team *tm)
 	 file_find_support_file(const_app("string_game_gui_boost_off_icon"), TRUE),
 	 file_find_support_file(const_app("string_game_gui_boost_on_icon"), TRUE)};
 
-    gtk_image_set_from_file(image_style_main, image_style_files[tm->style + 2]);
-    gtk_image_set_from_file(image_boost_main, image_boost_files[tm->boost + 1]);
+    gtk_image_set_from_file(style, image_style_files[tm->style + 2]);
+    gtk_image_set_from_file(boost, image_boost_files[tm->boost + 1]);    
+
+    for(i=0;i<5;i++)
+	g_free(image_style_files[i]);
+    for(i=0;i<3;i++)
+	g_free(image_boost_files[i]);
+}
+
+/** Set the images for the style and boost meters in the main
+    window and the live game window. */
+void
+game_gui_write_meters(const Team *tm)
+{
+    GtkImage *image_style_main = GTK_IMAGE(lookup_widget(window.main, "image_style")),
+	*image_boost_main = GTK_IMAGE(lookup_widget(window.main, "image_boost"));
+    GtkImage *image_style_live = NULL,
+	*image_boost_live = NULL;
+
+    game_gui_write_meter_images(tm, image_style_main, image_boost_main);
 
     if(window.live != NULL)
     {
 	image_style_live = GTK_IMAGE(lookup_widget(window.live, "image_lg_style"));
 	image_boost_live = GTK_IMAGE(lookup_widget(window.live, "image_lg_boost"));
 
-	gtk_image_set_from_file(image_style_live, image_style_files[tm->style + 2]);
-	gtk_image_set_from_file(image_boost_live, image_boost_files[tm->boost + 1]);
+	game_gui_write_meter_images(tm, image_style_live, image_boost_live);
     }
-
-    for(i=0;i<5;i++)
-	g_free(image_style_files[i]);
-    for(i=0;i<3;i++)
-	g_free(image_boost_files[i]);
 }
 
 /** Activate the appropriate radio items for
@@ -504,14 +569,13 @@ enum MainWindowInensitiveItems
     INSENSITIVE_ITEM_MENU_SCOUT,
     INSENSITIVE_ITEM_MENU_PHYSIO,
     INSENSITIVE_ITEM_MENU_BROWSE_TEAMS,
+    INSENSITIVE_ITEM_MENU_YOUTH_ACADEMY,
+    INSENSITIVE_ITEM_MENU_SHOW_JOB_EXCHANGE,
     INSENSITIVE_ITEM_MENU_PUT_ON_TRANSFER_LIST,
     INSENSITIVE_ITEM_MENU_REMOVE_FROM_TRANSFER_LIST,
     INSENSITIVE_ITEM_MENU_FIRE,
-    INSENSITIVE_ITEM_MENU_MANAGE_USERS,
-    INSENSITIVE_ITEM_MENU_USER_SHOW_LAST,
-    INSENSITIVE_ITEM_MENU_USER_SHOW_LAST_STATS,
-    INSENSITIVE_ITEM_MENU_USER_SHOW_COMING,
-    INSENSITIVE_ITEM_MENU_USER_SHOW_HISTORY,
+    INSENSITIVE_ITEM_MENU_MOVE_TO_YOUTH_ACADEMY,
+    INSENSITIVE_ITEM_MENU_USER,
     INSENSITIVE_ITEM_MENU_FINANCES_STADIUM,
     INSENSITIVE_ITEM_MENU_HELP,
     INSENSITIVE_ITEM_MENU_BROWSE_PLAYERS,
@@ -550,22 +614,20 @@ game_gui_set_main_window_sensitivity(gboolean value)
 	lookup_widget(window.main, "menu_physio");
     insensitive_items[INSENSITIVE_ITEM_MENU_BROWSE_TEAMS] = 
 	lookup_widget(window.main, "menu_browse_teams");
+    insensitive_items[INSENSITIVE_ITEM_MENU_YOUTH_ACADEMY] = 
+	lookup_widget(window.main, "menu_youth_academy");
+    insensitive_items[INSENSITIVE_ITEM_MENU_SHOW_JOB_EXCHANGE] = 
+	lookup_widget(window.main, "menu_show_job_exchange");
     insensitive_items[INSENSITIVE_ITEM_MENU_PUT_ON_TRANSFER_LIST] = 
 	lookup_widget(window.main, "menu_put_on_transfer_list");
     insensitive_items[INSENSITIVE_ITEM_MENU_REMOVE_FROM_TRANSFER_LIST] = 
 	lookup_widget(window.main, "menu_remove_from_transfer_list");
     insensitive_items[INSENSITIVE_ITEM_MENU_FIRE] = 
 	lookup_widget(window.main, "menu_fire");
-    insensitive_items[INSENSITIVE_ITEM_MENU_MANAGE_USERS] = 
-	lookup_widget(window.main, "menu_manage_users");
-    insensitive_items[INSENSITIVE_ITEM_MENU_USER_SHOW_LAST] = 
-	lookup_widget(window.main, "menu_user_show_last_match");
-    insensitive_items[INSENSITIVE_ITEM_MENU_USER_SHOW_LAST_STATS] = 
-	lookup_widget(window.main, "menu_user_show_last_stats");
-    insensitive_items[INSENSITIVE_ITEM_MENU_USER_SHOW_COMING] = 
-	lookup_widget(window.main, "menu_user_show_coming_matches");
-    insensitive_items[INSENSITIVE_ITEM_MENU_USER_SHOW_HISTORY] = 
-	lookup_widget(window.main, "menu_user_show_history");
+    insensitive_items[INSENSITIVE_ITEM_MENU_MOVE_TO_YOUTH_ACADEMY] = 
+	lookup_widget(window.main, "menu_move_to_youth_academy");
+    insensitive_items[INSENSITIVE_ITEM_MENU_USER] = 
+	lookup_widget(window.main, "menu_user");
     insensitive_items[INSENSITIVE_ITEM_MENU_FINANCES_STADIUM] = 
 	lookup_widget(window.main, "menu_finances_stadium");
     insensitive_items[INSENSITIVE_ITEM_MENU_HELP] = 
@@ -590,7 +652,7 @@ game_gui_set_main_window_sensitivity(gboolean value)
 	for(i=0;i<SHOW_ITEM_END;i++)
 	    gtk_widget_hide(show_items[i]);
 
-    gtk_widget_set_sensitive(window.main, value);
+    gtk_window_set_modal(GTK_WINDOW(window.live), !value);
 }
 
 /** Show a window with a warning.
@@ -616,15 +678,31 @@ game_gui_show_warning(const gchar *format, ...)
 }
 
 /** Show the job offer window.
-    @param team The team offering the job.
-    @param type The offer type (whether the user's been fired). */
+    @param team The team offering the job or NULL if we're looking
+    at a job offer from the job exchange.
+    @param job The job pointer or NULL (depends on whether we're looking
+    at a job offer from the job exchange).
+    @param type The offer type (eg. whether the user's been fired). */
 void
-game_gui_show_job_offer(Team *team, gint type)
+game_gui_show_job_offer(Team *team, Job *job, gint type)
 {
     gchar buf[SMALL], buf2[SMALL];    
     GtkLabel *label_text, *label_text2, *label_name, 
 	*label_league, *label_rank, *label_money,
 	*label_cap, *label_saf, *label_average_skill;
+    Team *tm = (type != STATUS_JOB_EXCHANGE_SHOW_TEAM) ?
+	team : job_get_team(job);
+
+    statp = (type != STATUS_JOB_EXCHANGE_SHOW_TEAM) ?
+	(gpointer)team : (gpointer)job;
+    stat2 = type;
+
+    if(type == STATUS_JOB_OFFER_FIRE_FINANCE ||
+       type == STATUS_JOB_OFFER_FIRE_FAILURE)
+	user_history_add(&current_user, (type == STATUS_JOB_OFFER_FIRE_FINANCE) ?
+			 USER_HISTORY_FIRE_FINANCE : USER_HISTORY_FIRE_FAILURE,
+			 current_user.tm->name, team->name, 
+			 league_cup_get_name_string(team->clid), NULL);
 
     window_create(WINDOW_JOB_OFFER);
 
@@ -641,38 +719,55 @@ game_gui_show_job_offer(Team *team, gint type)
 
     if(type == STATUS_JOB_OFFER_FIRE_FINANCE)
 	sprintf(buf, _("The team owners have fired you because of financial mismanagement. Luckily, the owners of %s have heard of your dismissal and offer you a job. Here's some information on %s:"),
-		team->name, team->name);
+		tm->name, tm->name);
     else if(type == STATUS_JOB_OFFER_FIRE_FAILURE)
 	sprintf(buf, _("The team owners have fired you because of unsuccessfulness. Luckily, the owners of %s have heard of your dismissal and offer you a job. Here's some information on %s:"),
-		team->name, team->name);
+		tm->name, tm->name);
     else if(type == STATUS_JOB_OFFER_SUCCESS)
 	sprintf(buf, _("The owners of %s are deeply impressed by your success with %s and would like to hire you. Here's some information on %s:"),
-		team->name, current_user.tm->name, team->name);
+		tm->name, current_user.tm->name, tm->name);
+    else if(type == STATUS_JOB_EXCHANGE_SHOW_TEAM)
+	strcpy(buf, _("Click on OK to apply for the job. Click on CANCEL to close the window."));
     
-    strcpy(buf2, _("Accept?"));
-    if(type != STATUS_JOB_OFFER_SUCCESS)
+    strcpy(buf2, (type != STATUS_JOB_EXCHANGE_SHOW_TEAM) ?
+	   _("Accept?") : _("Apply for the job?"));
+
+    if(type != STATUS_JOB_OFFER_SUCCESS &&
+       type != STATUS_JOB_EXCHANGE_SHOW_TEAM)
 	strcat(buf2, _(" (NOTE: If you don't, the game is over for you.)"));
 
     gtk_label_set_text(label_text, buf);
     gtk_label_set_text(label_text2, buf2);
-    gtk_label_set_text(label_name, team->name);
-    gtk_label_set_text(label_league, league_cup_get_name_string(team->clid));
-    gui_label_set_text_from_int(label_rank, team_get_league_rank(team), FALSE);
-    misc_print_grouped_int(math_round_integer(team->stadium.capacity * 
-					      math_rndi(const_int("int_initial_money_lower"),
-							const_int("int_initial_money_upper")), 2),
-			   buf);
-    gtk_label_set_text(label_money, buf);
-    misc_print_grouped_int(team->stadium.capacity, buf);
-    gtk_label_set_text(label_cap, buf);
-    gui_label_set_text_from_int(label_saf, (gint)rint(team->stadium.safety * 100), FALSE);
+    gtk_label_set_text(label_name, tm->name);
+    gtk_label_set_text(label_league, 
+		       (type != STATUS_JOB_EXCHANGE_SHOW_TEAM) ?
+		       league_cup_get_name_string(tm->clid) : job->league_name);
 
-    sprintf(buf, "%.1f", team_get_average_skill(team, FALSE));
+    if(job == NULL ||
+       job->type == JOB_TYPE_NATIONAL)
+	gui_label_set_text_from_int(label_rank, 
+				    team_get_league_rank(tm), FALSE);
+
+    misc_print_grouped_int(
+	math_round_integer(tm->stadium.capacity * 
+			   math_rndi(const_int("int_initial_money_lower"),
+				     const_int("int_initial_money_upper")), 2),
+	buf);
+
+    gtk_label_set_text(label_money, buf);
+    misc_print_grouped_int(tm->stadium.capacity, buf);
+    gtk_label_set_text(label_cap, buf);
+    gui_label_set_text_from_int(label_saf,
+				(gint)rint(tm->stadium.safety * 100), FALSE);
+
+    sprintf(buf, "%.1f", team_get_average_skill(tm, FALSE));
     gtk_label_set_text(label_average_skill, buf);
 
-    treeview_show_player_list_team(GTK_TREE_VIEW(lookup_widget(window.job_offer, "treeview_players")),
-				   team, 
-				   (type != STATUS_JOB_OFFER_SUCCESS) ? 2 : current_user.scout);
+    treeview_show_player_list_team(
+	GTK_TREE_VIEW(lookup_widget(window.job_offer, "treeview_players")),
+	tm, 
+	(type != STATUS_JOB_OFFER_SUCCESS && type != STATUS_JOB_EXCHANGE_SHOW_TEAM) ? 
+	2 : current_user.scout);
 }
 
 /** Write the checkbuttons in the menus. */
