@@ -1,3 +1,28 @@
+/*
+   league.c
+
+   Bygfoot Football Manager -- a small and simple GTK2-based
+   football management game.
+
+   http://bygfoot.sourceforge.net
+
+   Copyright (C) 2005  Gyözö Both (gyboth@bygfoot.com)
+
+   This program is free software; you can redistribute it and/or
+   modify it under the terms of the GNU General Public License
+   as published by the Free Software Foundation; either version 2
+   of the License, or (at your option) any later version.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+*/
+
 #include "cup.h"
 #include "free.h"
 #include "league.h"
@@ -27,7 +52,7 @@ league_new(gboolean new_id)
     new.sid = NULL;
     new.short_name = NULL;
     new.symbol = NULL;
-    
+
     new.id = (new_id) ? league_id_new : -1;
     new.layer = -1;
 
@@ -51,7 +76,7 @@ league_new(gboolean new_id)
     new.round_robins = 2;
     new.yellow_red = 1000;
 
-    new.stats = stat_league_new(new.id);
+    new.stats = stat_league_new("", "");
     
     new.active = TRUE;
 
@@ -102,10 +127,9 @@ league_cup_get_index_from_clid(gint clid)
 	    }
 
     if(index == -1)
-    {
-	g_warning("league_cup_get_index_from_clid: couldn't find league or cup with id %d\n", clid);
-	main_exit_program(EXIT_POINTER_NOT_FOUND, NULL);
-    }
+	main_exit_program(EXIT_POINTER_NOT_FOUND, 
+			  "league_cup_get_index_from_clid: couldn't find league or cup with id %d\n", 
+			  clid);
 
     return index;
 }
@@ -122,9 +146,9 @@ league_from_clid(gint clid)
 	if(lig(i).id == clid)
 	    return &lig(i);
 
-    g_warning("league_from_clid: didn't find league with id %d\n", clid);
-
-    main_exit_program(EXIT_POINTER_NOT_FOUND, NULL);
+    main_exit_program(EXIT_POINTER_NOT_FOUND, 
+		      "league_from_clid: didn't find league with id %d\n", 
+		      clid);
 
     return NULL;
 }
@@ -314,9 +338,8 @@ league_index_from_sid(const gchar *sid)
 	if(strcmp(lig(i).sid, sid) == 0)
 	    return i;
 
-    g_warning("league_index_from_sid: no index found for sid '%s'.\n", sid);
-
-    main_exit_program(EXIT_INT_NOT_FOUND, NULL);
+    main_exit_program(EXIT_INT_NOT_FOUND, 
+		      "league_index_from_sid: no index found for sid '%s'.\n", sid);
 
     return -1;
 }
@@ -350,7 +373,25 @@ league_season_start(League *league)
     gint idx = league_index_from_sid(league->sid);
     gboolean user_champ = 
 	(team_is_user(
-	    team_of_id(g_array_index(lig(0).table.elements, TableElement, 0).team_id)) != -1);
+	    team_of_id(
+		g_array_index(lig(0).table.elements, TableElement, 0).team_id)) != -1),
+	league_above_talent =
+	(team_get_average_talents(league->teams) > league->average_talent *
+	 const_float("float_season_end_league_above_talent_factor") && !user_champ),
+	team_is_top = FALSE;
+    
+    gfloat team_change_lower = 
+	const_float("float_season_end_team_change_lower"),
+	user_champ_addition = 
+	const_float("float_season_end_user_champ_addition"),
+	user_champ_best_teams_addition = 
+	const_float("float_season_end_user_champ_best_teams_addition"),
+	team_change_upper = 
+	const_float("float_season_end_team_change_upper");
+
+    gint user_champ_best_teams_limit =
+	const_int("int_season_end_user_champ_best_teams_limit");
+
     gfloat team_change_factor = 0;
 
     for(i=0;i<league->table.elements->len;i++)
@@ -365,26 +406,40 @@ league_season_start(League *league)
 	    g_array_index(league->table.elements, TableElement, i).values[j] = 0;
     }
 
+    /*d*/
+/*     if(league == &lig(0)) */
+/* 	printf("league %s av %.2f ab %d\n", league->name, */
+/* 	       team_get_average_talents(league->teams), league_above_talent); */
+
     for(i=0;i<league->teams->len;i++)
     {
+	team_is_top = 
+	    (team_get_league_rank(&g_array_index(league->teams, Team, i)) <= 
+	     user_champ_best_teams_limit && idx == 0 && user_champ);
+
 	team_change_factor = 
 	    (team_is_user(&g_array_index(league->teams, Team, i)) == -1) *
-	    math_rnd(const_float("float_season_end_team_change_lower") +
-		     (user_champ && idx == 0) * const_float("float_season_end_user_champ_addition"),
-		     const_float("float_season_end_team_change_upper") +
-		     (user_champ && idx == 0) * const_float("float_season_end_user_champ_addition"));
+	    math_rnd(team_change_lower + user_champ * user_champ_addition +
+		     team_is_top * user_champ_best_teams_addition -
+		     league_above_talent * (user_champ_addition / 2),
+		     team_change_upper + user_champ * user_champ_addition +
+		     team_is_top * user_champ_best_teams_addition -
+		     league_above_talent * (user_champ_addition / 2));
 
 	for(j=0;j<g_array_index(league->teams, Team, i).players->len;j++)
 	    player_season_start(
-		&g_array_index(g_array_index(league->teams, Team, i).players, Player, j), team_change_factor);
-
-	if(team_is_user(&g_array_index(league->teams, Team, i)) == -1)
-	    team_update_cpu_structure(&g_array_index(league->teams, Team, i));
+		&g_array_index(
+		    g_array_index(
+			league->teams, Team, i).players, Player, j), team_change_factor);
 
 	g_array_index(league->teams, Team, i).stadium.average_attendance =
 	    g_array_index(league->teams, Team, i).stadium.games = 
 	    g_array_index(league->teams, Team, i).stadium.possible_attendance = 0;
     }
+
+/*     if(league == &lig(0)) */
+/* 	printf("2 league %s av %.2f\n", league->name, */
+/* 	       team_get_average_talents(league->teams)); */
 }
 
 /** Find out whether the team with specified rank in the league
@@ -604,25 +659,25 @@ league_team_movements_print(const GArray *team_movements,
     gint i, j;
     const TeamMove *tmove = NULL;
 
-    printf("%-25s Dest \t\t Possible\n\n", "Team");
+    g_print("%-25s Dest \t\t Possible\n\n", "Team");
     for(i=0;i<team_movements->len;i++)
     {
 	tmove = &g_array_index(team_movements, TeamMove, i);
 	if(tmove->dest_assigned)
-	    printf("%-25s (%d) %s \t\t", tmove->tm.name,
+	    g_print("%-25s (%d) %s \t\t", tmove->tm.name,
 		   league_from_clid(tmove->tm.clid)->layer,
 		   lig(g_array_index(tmove->dest_idcs, gint, 0)).name);
 	else
-	    printf("%-25s (%d) UNASSIGNED \t\t", tmove->tm.name,
+	    g_print("%-25s (%d) UNASSIGNED \t\t", tmove->tm.name,
 		   league_from_clid(tmove->tm.clid)->layer);
 	for(j=0;j<tmove->dest_idcs->len;j++)
-	    printf("%d ", g_array_index(tmove->dest_idcs, gint, j));
-	printf("\n");
+	    g_print("%d ", g_array_index(tmove->dest_idcs, gint, j));
+	g_print("\n");
     }
 
-    printf("%-20s Size Cursize\n", "League");
+    g_print("%-20s Size Cursize\n", "League");
     for(i=0;i<ligs->len;i++)
-	printf("%-20s %d %d\n", lig(i).name, league_size[i],
+	g_print("%-20s %d %d\n", lig(i).name, league_size[i],
 	       league_cur_size[i]);
 }
 
@@ -655,7 +710,7 @@ league_team_movements_assign_dest(GArray *team_movements, gint idx,
     TeamMove *tmove = &g_array_index(team_movements, TeamMove, idx);
 
     if(debug > 60)
-	printf("league_team_movements_assign_dest %s\n", tmove->tm.name);
+	g_print("league_team_movements_assign_dest %s\n", tmove->tm.name);
 
     if(tmove->dest_idcs->len == 1)
 	dest_idx = g_array_index(tmove->dest_idcs, gint, 0);
@@ -670,11 +725,9 @@ league_team_movements_assign_dest(GArray *team_movements, gint idx,
     league_cur_size[dest_idx]++;
 
     if(league_cur_size[dest_idx] > league_size[dest_idx])
-    {
-	g_warning("league_team_movements_assign_dest: no room in league %s for team %s.",
-		  lig(dest_idx).name, tmove->tm.name);
-	main_exit_program(EXIT_PROM_REL, NULL);
-    }
+	main_exit_program(EXIT_PROM_REL, 
+			  "league_team_movements_assign_dest: no room in league %s for team %s.",
+			  lig(dest_idx).name, tmove->tm.name);
 
     tmove->dest_assigned = TRUE;
 
@@ -694,11 +747,9 @@ league_team_movements_assign_dest(GArray *team_movements, gint idx,
 			g_array_remove_index(tmove->dest_idcs, j);
 
 		if(tmove->dest_idcs->len == 0)
-		{
-		    g_warning("league_team_movements_assign_dest: no destinations left for team %s.",
-			      tmove->tm.name);
-		    main_exit_program(EXIT_PROM_REL, NULL);
-		}
+		    main_exit_program(EXIT_PROM_REL, 
+				      "league_team_movements_assign_dest: no destinations left for team %s.",
+				      tmove->tm.name);
 	    }
 	}
 
@@ -706,12 +757,14 @@ league_team_movements_assign_dest(GArray *team_movements, gint idx,
     {
 	if(tmove->prom_rel_type == PROM_REL_PROMOTION)
 	    user_history_add(&usr(team_is_user(&tmove->tm)),
-			     USER_HISTORY_PROMOTED, tmove->tm.id,
-			     lig(g_array_index(tmove->dest_idcs, gint, 0)).id, -1, "");
+			     USER_HISTORY_PROMOTED, tmove->tm.name,
+			     lig(g_array_index(tmove->dest_idcs, gint, 0)).name,
+			     NULL, NULL);
 	else
 	    user_history_add(&usr(team_is_user(&tmove->tm)),
-			     USER_HISTORY_RELEGATED, tmove->tm.id,
-			     lig(g_array_index(tmove->dest_idcs, gint, 0)).id, -1, "");
+			     USER_HISTORY_RELEGATED, tmove->tm.name,
+			     lig(g_array_index(tmove->dest_idcs, gint, 0)).name,
+			     NULL, NULL);
     }
 }
 
@@ -728,7 +781,7 @@ league_team_movements_prune(GArray *team_movements, const gint *league_size,
     gint i;
 
     if(debug > 60)
-	printf("league_team_movements_prune\n");
+	g_print("league_team_movements_prune\n");
 
     while(query_league_team_movements_unassigned_single(team_movements))
     {
@@ -751,7 +804,7 @@ league_team_movements_destinations(GArray *team_movements, const gint *league_si
     gint league_cur_size[ligs->len];
 
     if(debug > 60)
-	printf("league_team_movements_destinations\n");
+	g_print("league_team_movements_destinations\n");
 
     for(i=0;i<ligs->len;i++)
 	league_cur_size[i] = lig(i).teams->len;
