@@ -1,8 +1,35 @@
+/*
+   misc.c
+
+   Bygfoot Football Manager -- a small and simple GTK2-based
+   football management game.
+
+   http://bygfoot.sourceforge.net
+
+   Copyright (C) 2005  Gyözö Both (gyboth@bygfoot.com)
+
+   This program is free software; you can redistribute it and/or
+   modify it under the terms of the GNU General Public License
+   as published by the Free Software Foundation; either version 2
+   of the License, or (at your option) any later version.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+*/
+
 #include <time.h>
 
 #include "main.h"
 #include "maths.h"
 #include "misc.h"
+#include "option.h"
+#include "variables.h"
 
 /**
    Print the contents of a GError (if it was set).
@@ -14,14 +41,10 @@
 void
 misc_print_error(GError **error, gboolean abort_program)
 {
-    gchar buf[SMALL];
-
     if(*error == NULL)
 	return;
     
-    sprintf(buf, "%s", (*error)->message);
-
-    g_warning("error message: %s\n", buf);
+    g_warning("error message: %s\n", (*error)->message);
     g_error_free(*error);
     *error = NULL;
 
@@ -114,6 +137,24 @@ misc_randomise_g_pointer_array(GPtrArray *array)
     return new;
 }
 
+/** Return a freshly allocated copy of the array. */
+GPtrArray*
+misc_copy_ptr_array(const GPtrArray *array)
+{
+    gint i;
+    GPtrArray *copy = NULL;
+
+    if(array != NULL)
+	copy = g_ptr_array_new();
+    else
+	return NULL;
+
+    for(i=0;i<array->len;i++)
+	g_ptr_array_add(copy, g_ptr_array_index(array, i));
+
+    return copy;
+}
+
 /** Print a thousands-grouped output of 'number' into 'buf',
     like 2 234 345 instead of 2234345.
     @param number The number to print. 
@@ -227,53 +268,6 @@ query_misc_string_in_array(const gchar *string, GPtrArray *array)
     return FALSE;
 }
 
-/** Replace a token in a string by another string. 
-    The replacement should NOT CONTAIN THE TOKEN otherwise
-    we end up in an infinite loop. */
-void
-misc_string_replace_token(gchar *string, const gchar *token, const gchar *replacement)
-{
-    gchar buf[SMALL], buf2[SMALL];
-    gchar *occurrence = NULL;
-
-    if(strlen(string) < strlen(token))
-	return;
-
-    occurrence = g_strrstr(string, token);
-
-    if(occurrence != NULL)
-    {
-	strncpy(buf, string, strlen(string) - strlen(occurrence));
-	buf[strlen(string) - strlen(occurrence)] = '\0';
-	strcpy(buf2, occurrence + strlen(token));
-	sprintf(string, "%s%s%s", buf, replacement, buf2);
-	misc_string_replace_token(string, token, replacement);
-    }
-}
-
-/** Replace sums of the form [1 + 2 - 3] in the string. */
-void
-misc_replace_sums(gchar *string)
-{
-    gint i, result = -1;
-    gchar buf[SMALL], buf2[SMALL];
-    const gchar *buf_return = NULL;    
-
-    strcpy(buf, string);
-
-    for(i=0;i<strlen(buf);i++)
-	if(buf[i] == '[')
-	{
-	    strncpy(buf2, buf, i);
-	    buf2[i] = '\0';
-	    buf_return = misc_parse_expression(buf + i + 1, &result);
-	    sprintf(string, "%s%d%s", buf2, result, buf_return + 1);
-	    
-	    misc_replace_sums(string);
-	}
-}
-
-
 /** Get a float representation of someone's age
     based on his birth year and month. */
 gfloat
@@ -437,30 +431,248 @@ misc_parse(const gchar *s, gint *result)
    return s;
 }
 
-/** Return a freshly allocated copy of the array. */
-GPtrArray*
-misc_copy_ptr_array(const GPtrArray *array)
-{
-    gint i;
-    GPtrArray *copy = NULL;
-
-    if(array != NULL)
-	copy = g_ptr_array_new();
-    else
-	return NULL;
-
-    for(i=0;i<array->len;i++)
-	g_ptr_array_add(copy, g_ptr_array_index(array, i));
-
-    return copy;
-}
-
 /** Free the string if it's non-NULL and assign the contents to it. */
 void
 misc_string_assign(gchar **string, const gchar *contents)
 {
+    if(contents == NULL)
+	return;
+
     if(*string != NULL)
 	g_free(*string);
 
     *string = g_strdup(contents);
 }
+
+/** Choose one of strings separated with '|' */
+void
+misc_string_choose_random(gchar *string)
+{
+   const gchar STR_SEP = '|';
+   gint i = 0;
+   gint count = 1;
+   const gchar* start;
+   
+   for (i = 0; string[i]; i++)
+      count += (string[i] == STR_SEP);
+
+   if (count == 1)
+      return;
+      
+   count = math_rndi(0, count - 1) + 1;
+   start = string;
+   for (i = 0; string[i]; i++)
+      if (string[i] == STR_SEP)
+      {
+         count--;
+	 if (count == 1)
+	    start = string + i + 1;
+	 else if (!count)
+	 {
+	    string[i] = '\0';
+	    break;
+         }
+      }
+
+   strcpy(string, start);
+}
+
+/** Replace a token in a string by another string. 
+    The replacement should NOT CONTAIN THE TOKEN otherwise
+    we end up in an infinite loop. */
+void
+misc_string_replace_token(gchar *string, const gchar *token, const gchar *replacement)
+{
+    gchar buf[SMALL], buf2[SMALL];
+    gchar *occurrence = NULL;
+
+    occurrence = g_strrstr(string, token);
+
+    if(occurrence == NULL || strlen(string) < strlen(token))
+	return;
+    
+    while(occurrence != NULL)
+    {
+	strncpy(buf, string, strlen(string) - strlen(occurrence));
+	buf[strlen(string) - strlen(occurrence)] = '\0';
+	strcpy(buf2, occurrence + strlen(token));
+	sprintf(string, "%s%s%s", buf, replacement, buf2);
+	occurrence = g_strrstr(string, token);
+    }
+}
+
+/** Replace simple arithmetic expressions like "1 + 2"
+    and comparisons like "3 < 4" with the appropriate result. */
+void
+misc_string_replace_expressions(gchar *string)
+{
+    gint i, j, last_idx = 0;
+    gint value = -1;
+    gchar buf[SMALL], buf2[SMALL];
+    gchar *occurrence = NULL,
+	*occurrence2 = NULL;
+
+    if(debug > 100)
+	g_print("misc_string_replace_expressions: #%s#\n",
+		string);
+
+    while(g_strrstr(string, "[") != NULL)
+    {
+	strcpy(buf, string);
+	strcpy(string, "");
+
+	occurrence = g_strrstr(buf, "[");
+	i = strlen(buf) - strlen(occurrence);
+
+	strncpy(buf2, buf, i);
+	buf2[i] = '\0';
+	strcat(string, buf2);	
+
+	occurrence2 = g_strstr_len(occurrence, strlen(occurrence), "]");
+
+	if(occurrence2 == NULL)
+	{
+	    g_warning("misc_string_replace_expressions: no matching ] found.");
+	    return;
+	}
+
+	j = strlen(buf) - strlen(occurrence2);
+	
+	strncpy(buf2, buf + i + 1, j - i - 1);
+	buf2[j - i - 1] = '\0';
+	if (g_strrstr(buf2, "|"))
+	    misc_string_choose_random(buf2);
+	else 
+	{
+	    if(g_strrstr(buf2, "<") ||
+	       g_strrstr(buf2, ">") ||
+	       g_strrstr(buf2, "=") ||
+	       g_strrstr(buf2, " G ") ||
+	       g_strrstr(buf2, " L ") ||
+	       g_strrstr(buf2, " GE ") ||
+	       g_strrstr(buf2, " LE "))
+		misc_parse(buf2, &value);
+	    else
+		misc_parse_expression(buf2, &value);
+	    sprintf(buf2, "%d", value);
+	}
+	strcat(string, buf2);
+	value = -1;
+
+	last_idx = j + 1;
+
+	if(last_idx < strlen(buf))
+	{
+	    strncpy(buf2, buf + last_idx, strlen(buf) - last_idx);
+	    buf2[strlen(buf) - last_idx] = '\0';
+	    strcat(string, buf2);
+	}
+    }
+}
+
+/** Try to replace all special tokens in the string and write the result to dest.
+    @param string The string containing tokens.
+    @param token_rep The arrays with the tokens and replacements.
+    @return TRUE if we could replace all tokens and the commentary condition
+    was fulfilled, FALSE otherwise. */
+void
+misc_string_replace_tokens(gchar *string, GPtrArray **token_rep)
+{
+    gint i;
+
+    for(i=0;i<token_rep[0]->len;i++)
+	if(g_strrstr(string, 
+		     (gchar*)g_ptr_array_index(token_rep[0], i)))
+	    misc_string_replace_token(string, 
+				      (gchar*)g_ptr_array_index(token_rep[0], i),
+				      (gchar*)g_ptr_array_index(token_rep[1], i));
+}
+
+/** Replace the portion 'paren' plus parentheses in the string. */
+void
+misc_string_replace_parenthesised(gchar *string, const gchar *paren, 
+				  const gchar *replacement)
+{
+    gchar buf[SMALL];
+
+    sprintf(buf, "(%s)", paren);
+
+    misc_string_replace_token(string, buf, replacement);
+}
+
+/** Find the innermost parenthesised portion of the string
+    and write it into the dest string. */
+void
+misc_string_get_parenthesised(const gchar *string, gchar *dest)
+{
+    const gchar *openpar = g_strrstr(string, "(");
+    const gchar *closepar = g_strstr_len(string, strlen(string), ")");
+    gint len = strlen(openpar) - strlen(closepar) - 1;
+
+    strncpy(dest, openpar + 1, len);
+    dest[len] = '\0';
+}
+
+/** Find out whether the conditions in the string are fulfilled. */
+gboolean
+misc_parse_condition(const gchar *condition, GPtrArray **token_rep)
+{
+    gboolean return_value = FALSE;
+    gchar buf[SMALL], buf2[SMALL];
+
+    strcpy(buf, condition);
+
+    while(g_strrstr(buf, "("))
+    {
+	misc_string_get_parenthesised(buf, buf2);
+	if(misc_parse_condition(buf2, token_rep))
+	    misc_string_replace_parenthesised(buf, buf2, "1 = 1");
+	else
+	    misc_string_replace_parenthesised(buf, buf2, "1 = 2");
+    }
+
+    do
+    {
+	strcpy(buf2, buf);
+	misc_string_replace_tokens(buf, token_rep);
+	misc_string_replace_expressions(buf);
+    }
+    while(strcmp(buf2, buf) != 0);
+
+    if(g_strrstr(buf, "_") != NULL)
+	return FALSE;
+
+    misc_parse(buf, &return_value);
+
+    return return_value;
+}
+
+/** Add a replacement rule to the token array.
+    The string should be in allocated as it will
+    get freed later. */
+void
+misc_token_add(GPtrArray **token_rep, gint token_idx, 
+	       gchar *replacement)
+{
+    g_ptr_array_add(token_rep[0], 
+		    (gpointer)g_strdup(g_array_index(tokens.list, Option, token_idx).string_value));
+    g_ptr_array_add(token_rep[1], (gpointer)replacement);
+}
+
+/** Remove the replacement rule given by the index. */
+void
+misc_token_remove(GPtrArray **token_rep, gint idx)
+{
+    gint i;
+
+    for(i=token_rep[0]->len - 1; i >= 0; i--)
+	if(strcmp((gchar*)g_ptr_array_index(token_rep[0], i),
+		  g_array_index(tokens.list, Option, idx).string_value) == 0)
+	{
+	    g_free(g_ptr_array_index(token_rep[0], i));
+	    g_free(g_ptr_array_index(token_rep[1], i));
+	    
+	    g_ptr_array_remove_index_fast(token_rep[0], i);
+	    g_ptr_array_remove_index_fast(token_rep[1], i);
+	}    
+}
