@@ -1,3 +1,28 @@
+/*
+   misc_callbacks.c
+
+   Bygfoot Football Manager -- a small and simple GTK2-based
+   football management game.
+
+   http://bygfoot.sourceforge.net
+
+   Copyright (C) 2005  Gyözö Both (gyboth@bygfoot.com)
+
+   This program is free software; you can redistribute it and/or
+   modify it under the terms of the GNU General Public License
+   as published by the Free Software Foundation; either version 2
+   of the License, or (at your option) any later version.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+*/
+
 #include "callbacks.h"
 #include "callback_func.h"
 #include "game.h"
@@ -45,31 +70,12 @@ on_team_selection_cancel_clicked       (GtkButton       *button,
     main_exit_program(EXIT_OK, NULL);
 }
 
-
-void
-on_team_selection_tv_row_activated     (GtkTreeView     *treeview,
-                                        GtkTreePath     *path,
-                                        GtkTreeViewColumn *column,
-                                        gpointer         user_data)
-{
-    on_button_add_player_clicked(NULL, NULL);
-}
-
-void
-on_team_selection_load_clicked         (GtkButton       *button,
-                                        gpointer         user_data)
-{
-    stat5 = STATUS_LOAD_GAME_TEAM_SELECTION;
-    window_show_file_sel();
-}
-
-
 gboolean
 on_button_font_sel_cancel_clicked      (GtkWidget       *widget,
                                         GdkEvent        *event,
                                         gpointer         user_data)
 {
-    window_destroy(&window.font_sel, FALSE);
+    window_destroy(&window.font_sel);
 
     return FALSE;
 }
@@ -88,7 +94,7 @@ on_button_font_sel_ok_clicked          (GtkButton       *button,
 	g_free(font);
     }
 
-    window_destroy(&window.font_sel, FALSE);
+    window_destroy(&window.font_sel);
 }
 
 
@@ -139,7 +145,7 @@ on_button_live_close_clicked           (GtkButton       *button,
 	callback_show_next_live_game();
     else
     {
-	window_destroy(&window.live, TRUE);
+	window_destroy(&window.live);
 	stat1 = stat2 = stat3 = stat4 = -1;
     }
 }
@@ -242,7 +248,7 @@ void
 on_button_stadium_cancel_clicked       (GtkButton       *button,
                                         gpointer         user_data)
 {
-    window_destroy(&window.stadium, TRUE);
+    window_destroy(&window.stadium);
 }
 
 
@@ -260,13 +266,6 @@ on_spinbutton_capacity_button_press_event  (GtkWidget       *widget,
     return FALSE;
 }
 
-void
-on_button_startup_resume_clicked       (GtkButton       *button,
-                                        gpointer         user_data)
-{
-    misc_callback_startup_load("last_save");
-}
-
 gboolean
 on_window_stadium_delete_event         (GtkWidget       *widget,
                                         GdkEvent        *event,
@@ -328,7 +327,7 @@ on_button_sponsors_clicked             (GtkButton       *button,
 {
     misc_callback_new_sponsor();
 
-    window_destroy(&window.sponsors, FALSE);
+    window_destroy(&window.sponsors);
 }
 
 
@@ -367,7 +366,7 @@ on_button_sponsors_wait_clicked        (GtkButton       *button,
     else
 	current_user.counters[COUNT_USER_NEW_SPONSOR] = 0;
 
-    window_destroy(&window.sponsors, FALSE);
+    window_destroy(&window.sponsors);
 }
 
 gboolean
@@ -383,7 +382,10 @@ on_eventbox_lg_style_button_press_event (GtkWidget       *widget,
 	return FALSE;
     }
 
-    if(event->type != GDK_BUTTON_PRESS)
+    if(event->type != GDK_BUTTON_PRESS ||
+       g_array_index(usr(stat2).live_game.units, LiveGameUnit,
+		     usr(stat2).live_game.units->len -1).event.type == 
+       LIVE_GAME_EVENT_END_MATCH)
 	return FALSE;
 
     if(event->button == 3)
@@ -421,7 +423,10 @@ on_eventbox_lg_boost_button_press_event (GtkWidget       *widget,
 	return FALSE;
     }
 
-    if(event->type != GDK_BUTTON_PRESS)
+    if(event->type != GDK_BUTTON_PRESS ||
+       g_array_index(usr(stat2).live_game.units, LiveGameUnit,
+		     usr(stat2).live_game.units->len -1).event.type == 
+       LIVE_GAME_EVENT_END_MATCH)
 	return FALSE;
 
     if(event->button == 3)
@@ -450,3 +455,12 @@ on_eventbox_lg_boost_button_press_event (GtkWidget       *widget,
 
     return FALSE;
 }
+
+void
+on_button_team_selection_back_clicked  (GtkButton       *button,
+                                        gpointer         user_data)
+{
+    window_destroy(&window.startup);
+    stat0 = STATUS_SPLASH;
+    window_show_splash();
+}
