@@ -1,3 +1,28 @@
+/*
+   xml.h
+
+   Bygfoot Football Manager -- a small and simple GTK2-based
+   football management game.
+
+   http://bygfoot.sourceforge.net
+
+   Copyright (C) 2005  Gyözö Both (gyboth@bygfoot.com)
+
+   This program is free software; you can redistribute it and/or
+   modify it under the terms of the GNU General Public License
+   as published by the Free Software Foundation; either version 2
+   of the License, or (at your option) any later version.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+*/
+
 #ifndef XML_H
 #define XML_H
 
@@ -44,7 +69,9 @@ enum XmlTags
 #define TAG_END_PLAYERS 9900
 #define TAG_START_LEAGUE_STAT 10000
 #define TAG_START_SEASON_STATS 11000
+#define TAG_START_LEAGUES_CUPS 12000
 #define TAG_START_TRANSFERS 20000
+#define TAG_START_JOBS 21000
 
 #define xml_write_g_string(fil, gstring, tag, indent) xml_write_string(fil, (gstring)->str, tag, indent)
 
@@ -61,16 +88,10 @@ void
 xml_load_users(const gchar *dirname, const gchar *basename);
 
 void
-xml_load_leagues(const gchar *dirname, const gchar *basename);
+xml_load_league(const gchar *dirname, const gchar *basename);
 
 void
-xml_load_league(const gchar *dirname, const gchar *basename, const GPtrArray *dir_contents);
-
-void
-xml_load_cups(const gchar *dirname, const gchar *basename);
-
-void
-xml_load_cup(Cup *cup, const gchar *dirname, const gchar *basename, const GPtrArray *dir_contents);
+xml_load_cup(Cup *cup, const gchar *dirname, const gchar *basename);
 
 void
 xml_load_transfers(const gchar *dirname, const gchar *basename);
