@@ -1,3 +1,28 @@
+/*
+   xml_league.c
+
+   Bygfoot Football Manager -- a small and simple GTK2-based
+   football management game.
+
+   http://bygfoot.sourceforge.net
+
+   Copyright (C) 2005  Gyözö Both (gyboth@bygfoot.com)
+
+   This program is free software; you can redistribute it and/or
+   modify it under the terms of the GNU General Public License
+   as published by the Free Software Foundation; either version 2
+   of the License, or (at your option) any later version.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+*/
+
 #include "cup_struct.h"
 #include "file.h"
 #include "free.h"
@@ -108,7 +133,7 @@ xml_league_read_start_element (GMarkupParseContext *context,
 {
     PromRelElement new_element;
     Team new_team;
-    
+
     if(strcmp(element_name, TAG_LEAGUE) == 0)
     {
 	new_league = league_new(TRUE);
@@ -363,7 +388,7 @@ xml_league_read(const gchar *league_name, GArray *leagues)
 			    xml_league_read_text, NULL, NULL};
     GMarkupParseContext *context;
     gchar *file_contents;
-    gint length;
+    gsize length;
     GError *error = NULL;
     gchar buf[SMALL];
 
