@@ -1,3 +1,28 @@
+/*
+   xml_loadsave_cup.c
+
+   Bygfoot Football Manager -- a small and simple GTK2-based
+   football management game.
+
+   http://bygfoot.sourceforge.net
+
+   Copyright (C) 2005  Gyözö Both (gyboth@bygfoot.com)
+
+   This program is free software; you can redistribute it and/or
+   modify it under the terms of the GNU General Public License
+   as published by the Free Software Foundation; either version 2
+   of the License, or (at your option) any later version.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+*/
+
 #include "cup.h"
 #include "file.h"
 #include "misc.h"
@@ -217,7 +242,7 @@ xml_loadsave_cup_text         (GMarkupParseContext *context,
 	new_round.byes = int_value;
     else if(state == TAG_CUP_ROUND_TEAMS_FILE)
     {
-	sprintf(buf2, "%s/%s", dirname, buf);
+	sprintf(buf2, "%s%s%s", dirname, G_DIR_SEPARATOR_S, buf);
 	xml_loadsave_teams_read(buf2, new_round.teams);
 
 	for(i=0;i<new_round.teams->len;i++)
@@ -230,7 +255,7 @@ xml_loadsave_cup_text         (GMarkupParseContext *context,
     {
 	new_table = table_new();
 
-	sprintf(buf2, "%s/%s", dirname, buf);
+	sprintf(buf2, "%s%s%s", dirname, G_DIR_SEPARATOR_S, buf);
 	xml_loadsave_table_read(buf2, &new_table);
 	
 	g_array_append_val(new_round.tables, new_table);
@@ -257,7 +282,7 @@ xml_loadsave_cup_read(const gchar *filename, Cup *cup)
 			    xml_loadsave_cup_text, NULL, NULL};
     GMarkupParseContext *context;
     gchar *file_contents = NULL;
-    guint length;
+    gsize length;
     GError *error = NULL;
 
     context = 
