@@ -1,3 +1,28 @@
+/*
+   xml_loadsave_league.c
+
+   Bygfoot Football Manager -- a small and simple GTK2-based
+   football management game.
+
+   http://bygfoot.sourceforge.net
+
+   Copyright (C) 2005  Gyözö Both (gyboth@bygfoot.com)
+
+   This program is free software; you can redistribute it and/or
+   modify it under the terms of the GNU General Public License
+   as published by the Free Software Foundation; either version 2
+   of the License, or (at your option) any later version.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+*/
+
 #include "file.h"
 #include "league.h"
 #include "misc.h"
@@ -16,6 +41,7 @@ enum
     TAG_LEAGUE_FIRST_WEEK,
     TAG_LEAGUE_ROUND_ROBINS,
     TAG_LEAGUE_ACTIVE,
+    TAG_LEAGUE_AVERAGE_TALENT,
     TAG_LEAGUE_PROM_REL,
     TAG_LEAGUE_PROM_REL_PROM_GAMES_DEST_SID,
     TAG_LEAGUE_PROM_REL_PROM_GAMES_CUP_SID,
@@ -82,6 +108,7 @@ xml_loadsave_league_end_element    (GMarkupParseContext *context,
     if(tag == TAG_LEAGUE_FIRST_WEEK ||
        tag == TAG_LEAGUE_LAYER ||
        tag == TAG_LEAGUE_ACTIVE ||
+       tag == TAG_LEAGUE_AVERAGE_TALENT ||
        tag == TAG_LEAGUE_ROUND_ROBINS ||
        tag == TAG_NAME ||
        tag == TAG_NAMES_FILE ||
@@ -127,10 +154,12 @@ xml_loadsave_league_text         (GMarkupParseContext *context,
 {
     gchar buf[SMALL];
     gint int_value = -1;
+    gfloat float_value = -1;
 
     strncpy(buf, text, text_len);
     buf[text_len] = '\0';
 
+    float_value = (gfloat)g_ascii_strtod(buf, NULL) / 10000;
     int_value = (gint)g_ascii_strtod(buf, NULL);
 
     if(state == TAG_NAME)
@@ -157,6 +186,8 @@ xml_loadsave_league_text         (GMarkupParseContext *context,
 	new_league->yellow_red = int_value;
     else if(state == TAG_LEAGUE_ACTIVE)
 	new_league->active = int_value;
+    else if(state == TAG_LEAGUE_AVERAGE_TALENT)
+	new_league->average_talent = float_value;
     else if(state == TAG_LEAGUE_PROM_REL_PROM_GAMES_DEST_SID)
 	misc_string_assign(&new_league->prom_rel.prom_games_dest_sid, buf);
     else if(state == TAG_LEAGUE_PROM_REL_PROM_GAMES_CUP_SID)
@@ -181,7 +212,7 @@ xml_loadsave_league_read(const gchar *filename, League *league)
 			    xml_loadsave_league_text, NULL, NULL};
     GMarkupParseContext *context;
     gchar *file_contents;
-    guint length;
+    gsize length;
     GError *error = NULL;
 
     context = 
@@ -245,6 +276,7 @@ xml_loadsave_league_write(const gchar *prefix, const League *league)
     xml_write_int(fil, league->week_gap, TAG_WEEK_GAP, I0);
     xml_write_int(fil, league->yellow_red, TAG_YELLOW_RED, I0);
     xml_write_int(fil, league->active, TAG_LEAGUE_ACTIVE, I0);
+    xml_write_float(fil, league->average_talent, TAG_LEAGUE_AVERAGE_TALENT, I0);
 
     fprintf(fil, "%s<_%d>\n", I0, TAG_LEAGUE_PROM_REL);
 
