@@ -1,3 +1,28 @@
+/*
+   xml_loadsave_season_stats.c
+
+   Bygfoot Football Manager -- a small and simple GTK2-based
+   football management game.
+
+   http://bygfoot.sourceforge.net
+
+   Copyright (C) 2005  Gyözö Both (gyboth@bygfoot.com)
+
+   This program is free software; you can redistribute it and/or
+   modify it under the terms of the GNU General Public License
+   as published by the Free Software Foundation; either version 2
+   of the License, or (at your option) any later version.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+*/
+
 #include "file.h"
 #include "free.h"
 #include "misc.h"
@@ -124,8 +149,8 @@ xml_loadsave_season_stats_text         (GMarkupParseContext *context,
 	new_champ_stat.cl_name = g_strdup(buf);
     else if(state == TAG_LEAGUE_STAT_FILE)
     {
-	new_league_stat = stat_league_new(-1);
-	sprintf(buf2, "%s/%s", dirname2, buf);
+	new_league_stat = stat_league_new("", "");
+	sprintf(buf2, "%s%s%s", dirname2, G_DIR_SEPARATOR_S, buf);
 	xml_loadsave_league_stat_read(buf2, &new_league_stat);
 	g_array_append_val(new_season_stat.league_stats, new_league_stat);
     }
@@ -140,13 +165,13 @@ xml_loadsave_season_stats_read(const gchar *dirname, const gchar *prefix)
 			    xml_loadsave_season_stats_text, NULL, NULL};
     GMarkupParseContext *context;
     gchar *file_contents;
-    guint length;
+    gsize length;
     GError *error = NULL;
 
     context = 
 	g_markup_parse_context_new(&parser, 0, NULL, NULL);
 
-    sprintf(filename, "%s/%s___season_stats.xml", dirname, prefix);
+    sprintf(filename, "%s%s%s___season_stats.xml", dirname, G_DIR_SEPARATOR_S, prefix);
 
     if(!g_file_get_contents(filename, &file_contents, &length, &error))
     {
